@@ -30,6 +30,12 @@
 #include <gst/video/video.h>
 #include <gst/gst-i18n-plugin.h>
 
+static const GstElementDetails gst_pngdec_details =
+GST_ELEMENT_DETAILS ("PNG image decoder",
+    "Codec/Decoder/Image",
+    "Decode a png video frame to a raw image",
+    "Wim Taymans <wim@fluendo.com>");
+
 GST_DEBUG_CATEGORY_STATIC (pngdec_debug);
 #define GST_CAT_DEFAULT pngdec_debug
 
@@ -86,8 +92,7 @@ static GstStaticPadTemplate gst_pngdec_src_pad_template =
     GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_RGBA ";" GST_VIDEO_CAPS_RGB ";"
-        GST_VIDEO_CAPS_ARGB_64)
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_RGBA ";" GST_VIDEO_CAPS_RGB)
     );
 
 static GstStaticPadTemplate gst_pngdec_sink_pad_template =
@@ -102,21 +107,20 @@ gst_pngdec_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_pngdec_src_pad_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_pngdec_sink_pad_template);
-  gst_element_class_set_details_simple (element_class, "PNG image decoder",
-      "Codec/Decoder/Image",
-      "Decode a png video frame to a raw image",
-      "Wim Taymans <wim@fluendo.com>");
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_pngdec_src_pad_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_pngdec_sink_pad_template));
+  gst_element_class_set_details (element_class, &gst_pngdec_details);
 }
 
 static void
 gst_pngdec_class_init (GstPngDecClass * klass)
 {
+  GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
 
+  gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
@@ -187,7 +191,7 @@ user_info_callback (png_structp png_ptr, png_infop info)
   size_t buffer_size;
   GstBuffer *buffer = NULL;
 
-  pngdec = GST_PNGDEC (png_get_io_ptr (png_ptr));
+  pngdec = GST_PNGDEC (png_ptr->io_ptr);
 
   GST_LOG ("info ready");
 
@@ -199,14 +203,7 @@ user_info_callback (png_structp png_ptr, png_infop info)
 
   /* Allocate output buffer */
   pngdec->rowbytes = png_get_rowbytes (pngdec->png, pngdec->info);
-  if (pngdec->rowbytes > (G_MAXUINT32 - 3)
-      || pngdec->height > G_MAXUINT32 / pngdec->rowbytes) {
-    ret = GST_FLOW_ERROR;
-    goto beach;
-  }
-  pngdec->rowbytes = GST_ROUND_UP_4 (pngdec->rowbytes);
-  buffer_size = pngdec->height * pngdec->rowbytes;
-
+  buffer_size = pngdec->height * GST_ROUND_UP_4 (pngdec->rowbytes);
   ret =
       gst_pad_alloc_buffer_and_set_caps (pngdec->srcpad, GST_BUFFER_OFFSET_NONE,
       buffer_size, GST_PAD_CAPS (pngdec->srcpad), &buffer);
@@ -226,14 +223,14 @@ user_endrow_callback (png_structp png_ptr, png_bytep new_row,
 {
   GstPngDec *pngdec = NULL;
 
-  pngdec = GST_PNGDEC (png_get_io_ptr (png_ptr));
+  pngdec = GST_PNGDEC (png_ptr->io_ptr);
 
   /* FIXME: implement interlaced pictures */
 
   /* If buffer_out doesn't exist, it means buffer_alloc failed, which 
    * will already have set the return code */
   if (GST_IS_BUFFER (pngdec->buffer_out)) {
-    size_t offset = row_num * pngdec->rowbytes;
+    size_t offset = row_num * GST_ROUND_UP_4 (pngdec->rowbytes);
 
     GST_LOG ("got row %u, copying in buffer %p at offset %" G_GSIZE_FORMAT,
         (guint) row_num, pngdec->buffer_out, offset);
@@ -249,17 +246,15 @@ buffer_clip (GstPngDec * dec, GstBuffer * buffer)
   gboolean res = TRUE;
   gint64 cstart, cstop;
 
-
   if ((!GST_CLOCK_TIME_IS_VALID (GST_BUFFER_TIMESTAMP (buffer))) ||
       (!GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DURATION (buffer))) ||
       (dec->segment.format != GST_FORMAT_TIME))
     goto beach;
 
-  cstart = GST_BUFFER_TIMESTAMP (buffer);
-  cstop = GST_BUFFER_DURATION (buffer);
-
   if ((res = gst_segment_clip (&dec->segment, GST_FORMAT_TIME,
-              cstart, cstart + cstop, &cstart, &cstop))) {
+              GST_BUFFER_TIMESTAMP (buffer),
+              GST_BUFFER_TIMESTAMP (buffer) + GST_BUFFER_DURATION (buffer),
+              &cstart, &cstop))) {
     GST_BUFFER_TIMESTAMP (buffer) = cstart;
     GST_BUFFER_DURATION (buffer) = cstop - cstart;
   }
@@ -273,7 +268,7 @@ user_end_callback (png_structp png_ptr, png_infop info)
 {
   GstPngDec *pngdec = NULL;
 
-  pngdec = GST_PNGDEC (png_get_io_ptr (png_ptr));
+  pngdec = GST_PNGDEC (png_ptr->io_ptr);
 
   GST_LOG_OBJECT (pngdec, "and we are done reading this image");
 
@@ -306,23 +301,17 @@ user_read_data (png_structp png_ptr, png_bytep data, png_size_t length)
   GstPngDec *pngdec;
   GstBuffer *buffer;
   GstFlowReturn ret = GST_FLOW_OK;
-  guint size;
 
-  pngdec = GST_PNGDEC (png_get_io_ptr (png_ptr));
+  pngdec = GST_PNGDEC (png_ptr->io_ptr);
 
   GST_LOG ("reading %" G_GSIZE_FORMAT " bytes of data at offset %d", length,
       pngdec->offset);
 
   ret = gst_pad_pull_range (pngdec->sinkpad, pngdec->offset, length, &buffer);
-  if (ret != GST_FLOW_OK)
+  if ((ret != GST_FLOW_OK) || (GST_BUFFER_SIZE (buffer) != length))
     goto pause;
 
-  size = GST_BUFFER_SIZE (buffer);
-
-  if (size != length)
-    goto short_buffer;
-
-  memcpy (data, GST_BUFFER_DATA (buffer), size);
+  memcpy (data, GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer));
 
   gst_buffer_unref (buffer);
 
@@ -330,31 +319,14 @@ user_read_data (png_structp png_ptr, png_bytep data, png_size_t length)
 
   return;
 
-  /* ERRORS */
 pause:
-  {
-    GST_INFO_OBJECT (pngdec, "pausing task, reason %s",
-        gst_flow_get_name (ret));
-    gst_pad_pause_task (pngdec->sinkpad);
-    if (ret == GST_FLOW_UNEXPECTED) {
-      gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
-    } else if (ret < GST_FLOW_UNEXPECTED || ret == GST_FLOW_NOT_LINKED) {
-      GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
-          (_("Internal data stream error.")),
-          ("stream stopped, reason %s", gst_flow_get_name (ret)));
-      gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
-    }
-    png_error (png_ptr, "Internal data stream error.");
-    return;
-  }
-short_buffer:
-  {
-    gst_buffer_unref (buffer);
+  GST_INFO_OBJECT (pngdec, "pausing task, reason %s", gst_flow_get_name (ret));
+  gst_pad_pause_task (pngdec->sinkpad);
+  if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
+    gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
     GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
         (_("Internal data stream error.")),
-        ("Read %u, needed %" G_GSIZE_FORMAT "bytes", size, length));
-    ret = GST_FLOW_ERROR;
-    goto pause;
+        ("stream stopped, reason %s", gst_flow_get_name (ret)));
   }
 }
 
@@ -371,10 +343,12 @@ gst_pngdec_caps_create_and_set (GstPngDec * pngdec)
 
   /* Get bits per channel */
   bpc = png_get_bit_depth (pngdec->png, pngdec->info);
-  if (bpc > 8) {
-    /* Add alpha channel if 16-bit depth */
-    png_set_add_alpha (pngdec->png, 0xffff, PNG_FILLER_BEFORE);
-    png_set_swap (pngdec->png);
+
+  /* We don't handle 16 bits per color, strip down to 8 */
+  if (bpc == 16) {
+    GST_LOG_OBJECT (pngdec,
+        "this is a 16 bits per channel PNG image, strip down to 8 bits");
+    png_set_strip_16 (pngdec->png);
   }
 
   /* Get Color type */
@@ -396,11 +370,7 @@ gst_pngdec_caps_create_and_set (GstPngDec * pngdec)
     png_set_gray_to_rgb (pngdec->png);
     if (bpc < 8) {              /* Convert to 8 bits */
       GST_LOG_OBJECT (pngdec, "converting grayscale image to 8 bits");
-#if PNG_LIBPNG_VER < 10400
       png_set_gray_1_2_4_to_8 (pngdec->png);
-#else
-      png_set_expand_gray_1_2_4_to_8 (pngdec->png);
-#endif
     }
   }
 
@@ -427,11 +397,11 @@ gst_pngdec_caps_create_and_set (GstPngDec * pngdec)
   switch (pngdec->color_type) {
     case PNG_COLOR_TYPE_RGB:
       GST_LOG_OBJECT (pngdec, "we have no alpha channel, depth is 24 bits");
-      pngdec->bpp = 3 * bpc;
+      pngdec->bpp = 24;
       break;
     case PNG_COLOR_TYPE_RGB_ALPHA:
       GST_LOG_OBJECT (pngdec, "we have an alpha channel, depth is 32 bits");
-      pngdec->bpp = 4 * bpc;
+      pngdec->bpp = 32;
       break;
     default:
       GST_ELEMENT_ERROR (pngdec, STREAM, NOT_IMPLEMENTED, (NULL),
@@ -506,17 +476,13 @@ gst_pngdec_task (GstPad * pad)
 
   /* Allocate output buffer */
   rowbytes = png_get_rowbytes (pngdec->png, pngdec->info);
-  if (rowbytes > (G_MAXUINT32 - 3) || pngdec->height > G_MAXUINT32 / rowbytes) {
-    ret = GST_FLOW_ERROR;
-    goto pause;
-  }
-  rowbytes = GST_ROUND_UP_4 (rowbytes);
-  buffer_size = pngdec->height * rowbytes;
+  buffer_size = pngdec->height * GST_ROUND_UP_4 (rowbytes);
   ret =
       gst_pad_alloc_buffer_and_set_caps (pngdec->srcpad, GST_BUFFER_OFFSET_NONE,
       buffer_size, GST_PAD_CAPS (pngdec->srcpad), &buffer);
-  if (ret != GST_FLOW_OK)
+  if (ret != GST_FLOW_OK) {
     goto pause;
+  }
 
   rows = (png_bytep *) g_malloc (sizeof (png_bytep) * pngdec->height);
 
@@ -524,17 +490,18 @@ gst_pngdec_task (GstPad * pad)
 
   for (i = 0; i < pngdec->height; i++) {
     rows[i] = inp;
-    inp += rowbytes;
+    inp += GST_ROUND_UP_4 (rowbytes);
   }
 
   /* Read the actual picture */
   png_read_image (pngdec->png, rows);
-  g_free (rows);
+  free (rows);
 
   /* Push the raw RGB frame */
   ret = gst_pad_push (pngdec->srcpad, buffer);
-  if (ret != GST_FLOW_OK)
+  if (ret != GST_FLOW_OK) {
     goto pause;
+  }
 
   /* And we are done */
   gst_pad_pause_task (pngdec->sinkpad);
@@ -542,18 +509,13 @@ gst_pngdec_task (GstPad * pad)
   return;
 
 pause:
-  {
-    GST_INFO_OBJECT (pngdec, "pausing task, reason %s",
-        gst_flow_get_name (ret));
-    gst_pad_pause_task (pngdec->sinkpad);
-    if (ret == GST_FLOW_UNEXPECTED) {
-      gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
-    } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_UNEXPECTED) {
-      GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
-          (_("Internal data stream error.")),
-          ("stream stopped, reason %s", gst_flow_get_name (ret)));
-      gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
-    }
+  GST_INFO_OBJECT (pngdec, "pausing task, reason %s", gst_flow_get_name (ret));
+  gst_pad_pause_task (pngdec->sinkpad);
+  if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
+    GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
+        (_("Internal data stream error.")),
+        ("stream stopped, reason %s", gst_flow_get_name (ret)));
+    gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
   }
 }
 
@@ -873,6 +835,10 @@ setup_failed:
 static gboolean
 gst_pngdec_sink_activate_pull (GstPad * sinkpad, gboolean active)
 {
+  GstPngDec *pngdec;
+
+  pngdec = GST_PNGDEC (GST_OBJECT_PARENT (sinkpad));
+
   if (active) {
     return gst_pad_start_task (sinkpad, (GstTaskFunction) gst_pngdec_task,
         sinkpad);
