@@ -22,18 +22,14 @@
 /**
  * SECTION:element-dv1394src
  *
- * <refsect2>
- * <para>
  * Read DV (digital video) data from firewire port.
- * </para>
+ *
+ * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
- * gst-launch dv1394src ! dvdemux name=d ! queue ! dvdec ! xvimagesink d. ! queue ! alsasink
- * </programlisting>
- * This pipeline captures from the firewire port and displays it (might need
+ * |[
+ * gst-launch dv1394src ! queue ! dvdemux name=d ! queue ! dvdec ! xvimagesink d. ! queue ! alsasink
+ * ]| This pipeline captures from the firewire port and displays it (might need
  * format converters for audio/video).
- * </para>
  * </refsect2>
  */
 
@@ -68,8 +64,8 @@
 
 #define SEND_COMMAND(src, command)          \
 G_STMT_START {                              \
-  unsigned char c; c = command;             \
-  write (WRITE_SOCKET(src), &c, 1);         \
+  int _res; unsigned char c; c = command;   \
+  _res = write (WRITE_SOCKET(src), &c, 1);  \
 } G_STMT_END
 
 #define READ_COMMAND(src, command, res)        \
