@@ -25,6 +25,22 @@
  *       - make sure timestamps are correctly scaled everywhere
  */
 
+/**
+ * SECTION:element-matroskamux
+ *
+ * matroskamux muxes different input streams into a Matroska file.
+ *
+ * <refsect2>
+ * <title>Example launch line</title>
+ * |[
+ * gst-launch -v filesrc location=/path/to/mp3 ! mp3parse ! matroskamux name=mux ! filesink location=test.mkv  filesrc location=/path/to/theora.ogg ! oggdemux ! theoraparse ! mux.
+ * ]| This pipeline muxes an MP3 file and a Ogg Theora video into a Matroska file.
+ * |[
+ * gst-launch -v audiotestsrc num-buffers=100 ! audioconvert ! vorbisenc ! matroskamux ! filesink location=test.mka
+ * ]| This pipeline muxes a 440Hz sine wave encoded with the Vorbis codec into a Matroska file.
+ * </refsect2>
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -45,6 +61,9 @@ enum
   ARG_MATROSKA_VERSION
 };
 
+#define  DEFAULT_MATROSKA_VERSION        1
+#define  DEFAULT_WRITING_APP             "GStreamer Matroska muxer"
+
 static GstStaticPadTemplate src_templ = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -259,7 +278,7 @@ gst_matroska_mux_class_init (GstMatroskaMuxClass * klass)
   g_object_class_install_property (gobject_class, ARG_MATROSKA_VERSION,
       g_param_spec_int ("version", "Matroska version",
           "This parameter determines what matroska features can be used.",
-          1, 2, 1, G_PARAM_READWRITE));
+          1, 2, DEFAULT_MATROSKA_VERSION, G_PARAM_READWRITE));
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_matroska_mux_change_state);
@@ -291,11 +310,18 @@ gst_matroska_mux_init (GstMatroskaMux * mux, GstMatroskaMuxClass * g_class)
 
   mux->ebml_write = gst_ebml_write_new (mux->srcpad);
 
+  /* property defaults */
+  mux->matroska_version = DEFAULT_MATROSKA_VERSION;
+  mux->writing_app = g_strdup (DEFAULT_WRITING_APP);
+
   /* initialize internal variables */
   mux->index = NULL;
-  mux->matroska_version = 1;
+  mux->num_streams = 0;
+  mux->num_a_streams = 0;
+  mux->num_t_streams = 0;
+  mux->num_v_streams = 0;
 
-  /* Initialize all variables */
+  /* initialize remaining variables */
   gst_matroska_mux_reset (GST_ELEMENT (mux));
 }
 
@@ -354,20 +380,38 @@ gst_matroska_mux_create_uid (void)
   return uid;
 }
 
+
 /**
- * gst_matroska_pad_free:
+ * gst_matroska_pad_reset:
  * @collect_pad: the #GstMatroskaPad
  *
- * Release resources of a matroska collect pad.
+ * Reset and/or release resources of a matroska collect pad.
  */
 static void
-gst_matroska_pad_free (GstMatroskaPad * collect_pad)
+gst_matroska_pad_reset (GstMatroskaPad * collect_pad, gboolean full)
 {
+  gchar *name = NULL;
+  GstMatroskaTrackType type = 0;
+
   /* free track information */
   if (collect_pad->track != NULL) {
+    /* retrieve for optional later use */
+    name = collect_pad->track->name;
+    type = collect_pad->track->type;
+    /* extra for video */
+    if (type == GST_MATROSKA_TRACK_TYPE_VIDEO) {
+      GstMatroskaTrackVideoContext *ctx =
+          (GstMatroskaTrackVideoContext *) collect_pad->track;
+
+      if (ctx->dirac_unit) {
+        gst_buffer_unref (ctx->dirac_unit);
+        ctx->dirac_unit = NULL;
+      }
+    }
     g_free (collect_pad->track->codec_id);
     g_free (collect_pad->track->codec_name);
-    g_free (collect_pad->track->name);
+    if (full)
+      g_free (collect_pad->track->name);
     g_free (collect_pad->track->language);
     g_free (collect_pad->track->codec_priv);
     g_free (collect_pad->track);
@@ -379,6 +423,51 @@ gst_matroska_pad_free (GstMatroskaPad * collect_pad)
     gst_buffer_unref (collect_pad->buffer);
     collect_pad->buffer = NULL;
   }
+
+  if (!full && type != 0) {
+    GstMatroskaTrackContext *context;
+
+    /* create a fresh context */
+    switch (type) {
+      case GST_MATROSKA_TRACK_TYPE_VIDEO:
+        context = (GstMatroskaTrackContext *)
+            g_new0 (GstMatroskaTrackVideoContext, 1);
+        break;
+      case GST_MATROSKA_TRACK_TYPE_AUDIO:
+        context = (GstMatroskaTrackContext *)
+            g_new0 (GstMatroskaTrackAudioContext, 1);
+        break;
+      case GST_MATROSKA_TRACK_TYPE_SUBTITLE:
+        context = (GstMatroskaTrackContext *)
+            g_new0 (GstMatroskaTrackSubtitleContext, 1);
+        break;
+      default:
+        g_assert_not_reached ();
+        break;
+    }
+
+    context->type = type;
+    context->name = name;
+    /* TODO: check default values for the context */
+    context->flags = GST_MATROSKA_TRACK_ENABLED | GST_MATROSKA_TRACK_DEFAULT;
+    collect_pad->track = context;
+    collect_pad->buffer = NULL;
+    collect_pad->duration = 0;
+    collect_pad->start_ts = GST_CLOCK_TIME_NONE;
+    collect_pad->end_ts = GST_CLOCK_TIME_NONE;
+  }
+}
+
+/**
+ * gst_matroska_pad_free:
+ * @collect_pad: the #GstMatroskaPad
+ *
+ * Release resources of a matroska collect pad.
+ */
+static void
+gst_matroska_pad_free (GstMatroskaPad * collect_pad)
+{
+  gst_matroska_pad_reset (collect_pad, TRUE);
 }
 
 
@@ -401,29 +490,17 @@ gst_matroska_mux_reset (GstElement * element)
   mux->state = GST_MATROSKA_MUX_STATE_START;
 
   /* clean up existing streams */
-  while ((walk = mux->collect->data) != NULL) {
+
+  for (walk = mux->collect->data; walk; walk = g_slist_next (walk)) {
     GstMatroskaPad *collect_pad;
     GstPad *thepad;
 
     collect_pad = (GstMatroskaPad *) walk->data;
     thepad = collect_pad->collect.pad;
 
-    /* free collect pad resources */
-    gst_matroska_pad_free (collect_pad);
-
-    /* remove from collectpads */
-    gst_collect_pads_remove_pad (mux->collect, thepad);
+    /* reset collect pad to pristine state */
+    gst_matroska_pad_reset (collect_pad, FALSE);
   }
-  mux->num_streams = 0;
-  mux->num_a_streams = 0;
-  mux->num_t_streams = 0;
-  mux->num_v_streams = 0;
-
-  /* reset writing_app */
-  if (mux->writing_app) {
-    g_free (mux->writing_app);
-  }
-  mux->writing_app = g_strdup ("GStreamer Matroska muxer");
 
   /* reset indexes */
   mux->num_indexes = 0;
@@ -440,10 +517,7 @@ gst_matroska_mux_reset (GstElement * element)
   mux->cluster_pos = 0;
 
   /* reset tags */
-  if (mux->tags) {
-    gst_tag_list_free (mux->tags);
-    mux->tags = NULL;
-  }
+  gst_tag_setter_reset_tags (GST_TAG_SETTER (mux));
 }
 
 /**
@@ -495,7 +569,9 @@ gst_matroska_mux_handle_sink_event (GstPad * pad, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_TAG:
+      GST_DEBUG_OBJECT (mux, "received tag event");
       gst_event_parse_tag (event, &list);
+
       collect_pad = (GstMatroskaPad *) gst_pad_get_element_private (pad);
       g_assert (collect_pad);
       context = collect_pad->track;
@@ -505,12 +581,8 @@ gst_matroska_mux_handle_sink_event (GstPad * pad, GstEvent * event)
        * 639-2 according to matroska specs, but it will have to do for now */
       gst_tag_list_get_string (list, GST_TAG_LANGUAGE_CODE, &context->language);
 
-      if (mux->tags) {
-        gst_tag_list_insert (mux->tags, list, GST_TAG_MERGE_PREPEND);
-      } else {
-        mux->tags = gst_tag_list_copy (list);
-      }
-
+      gst_tag_setter_merge_tags (GST_TAG_SETTER (mux), list,
+          gst_tag_setter_get_tag_merge_mode (GST_TAG_SETTER (mux)));
       break;
     case GST_EVENT_NEWSEGMENT:
       /* We don't support NEWSEGMENT events */
@@ -548,6 +620,8 @@ gst_matroska_mux_video_pad_setcaps (GstPad * pad, GstCaps * caps)
   GstMatroskaPad *collect_pad;
   GstStructure *structure;
   const gchar *mimetype;
+  const GValue *value = NULL;
+  const GstBuffer *codec_buf = NULL;
   gint width, height, pixel_width, pixel_height;
   gint fps_d, fps_n;
 
@@ -611,6 +685,11 @@ skip_details:
    *       - add new formats
    */
 
+  /* extract codec_data, may turn out needed */
+  value = gst_structure_get_value (structure, "codec_data");
+  if (value)
+    codec_buf = gst_value_get_buffer (value);
+
   /* find type */
   if (!strcmp (mimetype, "video/x-raw-yuv")) {
     context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_UNCOMPRESSED);
@@ -625,56 +704,72 @@ skip_details:
       ||!strcmp (mimetype, "video/x-huffyuv")
       || !strcmp (mimetype, "video/x-divx")
       || !strcmp (mimetype, "video/x-dv")
-      || !strcmp (mimetype, "video/x-h263")) {
+      || !strcmp (mimetype, "video/x-h263")
+      || !strcmp (mimetype, "video/x-msmpeg")) {
     BITMAPINFOHEADER *bih;
-    const GValue *codec_data;
     gint size = sizeof (BITMAPINFOHEADER);
-
-    bih = g_new0 (BITMAPINFOHEADER, 1);
-    GST_WRITE_UINT32_LE (&bih->bi_size, size);
-    GST_WRITE_UINT32_LE (&bih->bi_width, videocontext->pixel_width);
-    GST_WRITE_UINT32_LE (&bih->bi_height, videocontext->pixel_height);
-    GST_WRITE_UINT16_LE (&bih->bi_planes, (guint16) 1);
-    GST_WRITE_UINT16_LE (&bih->bi_bit_count, (guint16) 24);
-    GST_WRITE_UINT32_LE (&bih->bi_size_image, videocontext->pixel_width *
-        videocontext->pixel_height * 3);
+    guint32 fourcc = 0;
 
     if (!strcmp (mimetype, "video/x-xvid"))
-      GST_WRITE_UINT32_LE (&bih->bi_compression, GST_STR_FOURCC ("XVID"));
+      fourcc = GST_MAKE_FOURCC ('X', 'V', 'I', 'D');
     else if (!strcmp (mimetype, "video/x-huffyuv"))
-      GST_WRITE_UINT32_LE (&bih->bi_compression, GST_STR_FOURCC ("HFYU"));
+      fourcc = GST_MAKE_FOURCC ('H', 'F', 'Y', 'U');
     else if (!strcmp (mimetype, "video/x-dv"))
-      GST_WRITE_UINT32_LE (&bih->bi_compression, GST_STR_FOURCC ("DVSD"));
+      fourcc = GST_MAKE_FOURCC ('D', 'V', 'S', 'D');
     else if (!strcmp (mimetype, "video/x-h263"))
-      GST_WRITE_UINT32_LE (&bih->bi_compression, GST_STR_FOURCC ("H263"));
+      fourcc = GST_MAKE_FOURCC ('H', '2', '6', '3');
     else if (!strcmp (mimetype, "video/x-divx")) {
       gint divxversion;
 
       gst_structure_get_int (structure, "divxversion", &divxversion);
       switch (divxversion) {
         case 3:
-          GST_WRITE_UINT32_LE (&bih->bi_compression, GST_STR_FOURCC ("DIV3"));
+          fourcc = GST_MAKE_FOURCC ('D', 'I', 'V', '3');
           break;
         case 4:
-          GST_WRITE_UINT32_LE (&bih->bi_compression, GST_STR_FOURCC ("DIVX"));
+          fourcc = GST_MAKE_FOURCC ('D', 'I', 'V', 'X');
           break;
         case 5:
-          GST_WRITE_UINT32_LE (&bih->bi_compression, GST_STR_FOURCC ("DX50"));
+          fourcc = GST_MAKE_FOURCC ('D', 'X', '5', '0');
+          break;
+      }
+    } else if (!strcmp (mimetype, "video/x-msmpeg")) {
+      gint msmpegversion;
+
+      gst_structure_get_int (structure, "msmpegversion", &msmpegversion);
+      switch (msmpegversion) {
+        case 41:
+          fourcc = GST_MAKE_FOURCC ('M', 'P', 'G', '4');
+          break;
+        case 42:
+          fourcc = GST_MAKE_FOURCC ('M', 'P', '4', '2');
+          break;
+        case 43:
+          goto msmpeg43;
           break;
       }
     }
 
-    /* process codec private/initialization data, if any */
-    codec_data = gst_structure_get_value (structure, "codec_data");
-    if (codec_data) {
-      GstBuffer *codec_data_buf;
+    if (!fourcc)
+      return FALSE;
 
-      codec_data_buf = g_value_peek_pointer (codec_data);
-      size += GST_BUFFER_SIZE (codec_data_buf);
+    bih = g_new0 (BITMAPINFOHEADER, 1);
+    GST_WRITE_UINT32_LE (&bih->bi_size, size);
+    GST_WRITE_UINT32_LE (&bih->bi_width, videocontext->pixel_width);
+    GST_WRITE_UINT32_LE (&bih->bi_height, videocontext->pixel_height);
+    GST_WRITE_UINT32_LE (&bih->bi_compression, fourcc);
+    GST_WRITE_UINT16_LE (&bih->bi_planes, (guint16) 1);
+    GST_WRITE_UINT16_LE (&bih->bi_bit_count, (guint16) 24);
+    GST_WRITE_UINT32_LE (&bih->bi_size_image, videocontext->pixel_width *
+        videocontext->pixel_height * 3);
+
+    /* process codec private/initialization data, if any */
+    if (codec_buf) {
+      size += GST_BUFFER_SIZE (codec_buf);
       bih = g_realloc (bih, size);
       GST_WRITE_UINT32_LE (&bih->bi_size, size);
       memcpy ((guint8 *) bih + sizeof (BITMAPINFOHEADER),
-          GST_BUFFER_DATA (codec_data_buf), GST_BUFFER_SIZE (codec_data_buf));
+          GST_BUFFER_DATA (codec_buf), GST_BUFFER_SIZE (codec_buf));
     }
 
     context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_VFW_FOURCC);
@@ -683,8 +778,6 @@ skip_details:
 
     return TRUE;
   } else if (!strcmp (mimetype, "video/x-h264")) {
-    const GValue *codec_data;
-
     context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_MPEG4_AVC);
 
     if (context->codec_priv != NULL) {
@@ -693,22 +786,12 @@ skip_details:
       context->codec_priv_size = 0;
     }
 
-
     /* Create avcC header */
-    codec_data = gst_structure_get_value (structure, "codec_data");
-
-    if (codec_data != NULL) {
-      guint8 *priv_data = NULL;
-      guint priv_data_size = 0;
-      GstBuffer *codec_data_buf = g_value_peek_pointer (codec_data);
-
-      priv_data_size = GST_BUFFER_SIZE (codec_data_buf);
-      priv_data = g_malloc0 (priv_data_size);
-
-      memcpy (priv_data, GST_BUFFER_DATA (codec_data_buf), priv_data_size);
-
-      context->codec_priv = priv_data;
-      context->codec_priv_size = priv_data_size;
+    if (codec_buf != NULL) {
+      context->codec_priv_size = GST_BUFFER_SIZE (codec_buf);
+      context->codec_priv = g_malloc0 (context->codec_priv_size);
+      memcpy (context->codec_priv, GST_BUFFER_DATA (codec_buf),
+          context->codec_priv_size);
     }
 
     return TRUE;
@@ -752,8 +835,18 @@ skip_details:
         return FALSE;
     }
 
+    /* global headers may be in codec data */
+    if (codec_buf != NULL) {
+      context->codec_priv_size = GST_BUFFER_SIZE (codec_buf);
+      context->codec_priv = g_malloc0 (context->codec_priv_size);
+      memcpy (context->codec_priv, GST_BUFFER_DATA (codec_buf),
+          context->codec_priv_size);
+    }
+
     return TRUE;
   } else if (!strcmp (mimetype, "video/x-msmpeg")) {
+  msmpeg43:
+    /* can only make it here if preceding case verified it was version 3 */
     context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_MSMPEG4V3);
 
     return TRUE;
@@ -1037,7 +1130,7 @@ flac_streamheader_to_codecdata (const GValue * streamheader,
     context->codec_priv =
         g_realloc (context->codec_priv,
         context->codec_priv_size + GST_BUFFER_SIZE (buffer));
-    memcpy (context->codec_priv + context->codec_priv_size,
+    memcpy ((guint8 *) context->codec_priv + context->codec_priv_size,
         GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer));
     context->codec_priv_size =
         context->codec_priv_size + GST_BUFFER_SIZE (buffer);
@@ -1046,6 +1139,42 @@ flac_streamheader_to_codecdata (const GValue * streamheader,
   return TRUE;
 }
 
+static gchar *
+aac_codec_data_to_codec_id (const GstBuffer * buf)
+{
+  gchar *result;
+  gint profile;
+
+  /* default to MAIN */
+  profile = 1;
+
+  if (GST_BUFFER_SIZE (buf) >= 2) {
+    profile = GST_READ_UINT8 (GST_BUFFER_DATA (buf));
+    profile >>= 3;
+  }
+
+  switch (profile) {
+    case 1:
+      result = "MAIN";
+      break;
+    case 2:
+      result = "LC";
+      break;
+    case 3:
+      result = "SSR";
+      break;
+    case 4:
+      result = "LTP";
+      break;
+    default:
+      GST_WARNING ("unknown AAC profile, defaulting to MAIN");
+      result = "MAIN";
+      break;
+  }
+
+  return result;
+}
+
 /**
  * gst_matroska_mux_audio_pad_setcaps:
  * @pad: Pad which got the caps.
@@ -1095,6 +1224,12 @@ gst_matroska_mux_audio_pad_setcaps (GstPad * pad, GstCaps * caps)
 
   if (!strcmp (mimetype, "audio/mpeg")) {
     gint mpegversion = 0;
+    const GValue *codec_data;
+    const GstBuffer *buf = NULL;
+
+    codec_data = gst_structure_get_value (structure, "codec_data");
+    if (codec_data)
+      buf = gst_value_get_buffer (codec_data);
 
     gst_structure_get_int (structure, "mpegversion", &mpegversion);
     switch (mpegversion) {
@@ -1139,12 +1274,24 @@ gst_matroska_mux_audio_pad_setcaps (GstPad * pad, GstCaps * caps)
         break;
       }
       case 2:
-        context->codec_id =
-            g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_AAC_MPEG2 "MAIN");
+        if (buf) {
+          context->codec_id =
+              g_strdup_printf (GST_MATROSKA_CODEC_ID_AUDIO_AAC_MPEG2 "%s",
+              aac_codec_data_to_codec_id (buf));
+        } else {
+          GST_DEBUG_OBJECT (mux, "no AAC codec_data; not packetized");
+          return FALSE;
+        }
         break;
       case 4:
-        context->codec_id =
-            g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_AAC_MPEG4 "MAIN");
+        if (buf) {
+          context->codec_id =
+              g_strdup_printf (GST_MATROSKA_CODEC_ID_AUDIO_AAC_MPEG4 "%s",
+              aac_codec_data_to_codec_id (buf));
+        } else {
+          GST_DEBUG_OBJECT (mux, "no AAC codec_data; not packetized");
+          return FALSE;
+        }
         break;
       default:
         return FALSE;
@@ -1362,14 +1509,12 @@ gst_matroska_mux_request_new_pad (GstElement * element,
   newpad = gst_pad_new_from_template (templ, name);
   g_free (name);
   collect_pad = (GstMatroskaPad *)
-      gst_collect_pads_add_pad (mux->collect, newpad, sizeof (GstMatroskaPad));
+      gst_collect_pads_add_pad_full (mux->collect, newpad,
+      sizeof (GstMatroskaPad),
+      (GstCollectDataDestroyNotify) gst_matroska_pad_free);
 
-  /* TODO: check default values for the context */
-  context->flags = GST_MATROSKA_TRACK_ENABLED | GST_MATROSKA_TRACK_DEFAULT;
   collect_pad->track = context;
-  collect_pad->buffer = NULL;
-  collect_pad->start_ts = GST_CLOCK_TIME_NONE;
-  collect_pad->end_ts = GST_CLOCK_TIME_NONE;
+  gst_matroska_pad_reset (collect_pad, FALSE);
 
   /* FIXME: hacked way to override/extend the event function of
    * GstCollectPads; because it sets its own event function giving the
@@ -1424,15 +1569,13 @@ gst_matroska_mux_release_pad (GstElement * element, GstPad * pad)
           mux->duration < collect_pad->duration)
         mux->duration = collect_pad->duration;
 
-      gst_matroska_pad_free (collect_pad);
-      gst_collect_pads_remove_pad (mux->collect, pad);
-      gst_element_remove_pad (element, pad);
-      mux->num_streams--;
-      return;
+      break;
     }
   }
 
-  g_warning ("%s: unknown pad %s", GST_FUNCTION, GST_PAD_NAME (pad));
+  gst_collect_pads_remove_pad (mux->collect, pad);
+  if (gst_element_remove_pad (element, pad))
+    mux->num_streams--;
 }
 
 
@@ -1724,7 +1867,7 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
   guint64 pos;
   guint64 duration = 0;
   GSList *collected;
-  GstTagList *tags;
+  const GstTagList *tags;
 
   /* finish last cluster */
   if (mux->cluster) {
@@ -1761,10 +1904,9 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
   }
 
   /* tags */
-  tags = gst_tag_list_merge (gst_tag_setter_get_tag_list (GST_TAG_SETTER (mux)),
-      mux->tags, GST_TAG_MERGE_APPEND);
+  tags = gst_tag_setter_get_tag_list (GST_TAG_SETTER (mux));
 
-  if (tags != NULL) {
+  if (tags != NULL && !gst_tag_list_is_empty (tags)) {
     guint64 master_tags, master_tag;
 
     GST_DEBUG ("Writing tags");
@@ -1776,7 +1918,6 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
     gst_tag_list_foreach (tags, gst_matroska_mux_write_simple_tag, ebml);
     gst_ebml_write_master_finish (ebml, master_tag);
     gst_ebml_write_master_finish (ebml, master_tags);
-    gst_tag_list_free (tags);
   }
 
   /* update seekhead. We know that:
@@ -1942,6 +2083,68 @@ gst_matroska_mux_create_buffer_header (GstMatroskaTrackContext * track,
   return hdr;
 }
 
+static GstBuffer *
+gst_matroska_mux_handle_dirac_packet (GstMatroskaMux * mux,
+    GstMatroskaPad * collect_pad, GstBuffer * buf)
+{
+  GstMatroskaTrackVideoContext *ctx =
+      (GstMatroskaTrackVideoContext *) collect_pad->track;
+  const guint8 *data = GST_BUFFER_DATA (buf);
+  guint size = GST_BUFFER_SIZE (buf);
+  guint8 parse_code;
+  guint32 next_parse_offset;
+  GstBuffer *ret = NULL;
+  gboolean is_picture = FALSE;
+
+  if (GST_BUFFER_SIZE (buf) < 13) {
+    gst_buffer_unref (buf);
+    return ret;
+  }
+
+  /* Check if this buffer contains a picture packet */
+  while (size >= 13) {
+    if (GST_READ_UINT32_BE (data) != 0x42424344) {
+      gst_buffer_unref (buf);
+      return ret;
+    }
+
+    parse_code = GST_READ_UINT8 (data + 4);
+    if (parse_code == 0x00) {
+      if (ctx->dirac_unit) {
+        gst_buffer_unref (ctx->dirac_unit);
+        ctx->dirac_unit = NULL;
+      }
+    } else if (parse_code & 0x08) {
+      is_picture = TRUE;
+      break;
+    }
+
+    next_parse_offset = GST_READ_UINT32_BE (data + 5);
+
+    data += next_parse_offset;
+    size -= next_parse_offset;
+  }
+
+  if (ctx->dirac_unit)
+    ctx->dirac_unit = gst_buffer_join (ctx->dirac_unit, gst_buffer_ref (buf));
+  else
+    ctx->dirac_unit = gst_buffer_ref (buf);
+
+  if (is_picture) {
+    ret = gst_buffer_make_metadata_writable (ctx->dirac_unit);
+    ctx->dirac_unit = NULL;
+    gst_buffer_copy_metadata (ret, buf,
+        GST_BUFFER_COPY_FLAGS | GST_BUFFER_COPY_TIMESTAMPS |
+        GST_BUFFER_COPY_CAPS);
+    gst_buffer_unref (buf);
+  } else {
+    gst_buffer_unref (buf);
+    ret = NULL;
+  }
+
+  return ret;
+}
+
 /**
  * gst_matroska_mux_write_data:
  * @mux: #GstMatroskaMux
@@ -1975,6 +2178,15 @@ gst_matroska_mux_write_data (GstMatroskaMux * mux, GstMatroskaPad * collect_pad)
     return GST_FLOW_OK;
   }
 
+  /* for dirac we have to queue up everything up to a picture unit */
+  if (collect_pad->track->codec_id != NULL &&
+      strcmp (collect_pad->track->codec_id,
+          GST_MATROSKA_CODEC_ID_VIDEO_DIRAC) == 0) {
+    buf = gst_matroska_mux_handle_dirac_packet (mux, collect_pad, buf);
+    if (!buf)
+      return GST_FLOW_OK;
+  }
+
   /* hm, invalid timestamp (due to --to be fixed--- element upstream);
    * this would wreak havoc with time stored in matroska file */
   /* TODO: maybe calculate a timestamp by using the previous timestamp
