@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2005> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,6 +27,9 @@
 #include <string.h>
 #include "gstrtpamrdepay.h"
 
+GST_DEBUG_CATEGORY_STATIC (rtpamrdepay_debug);
+#define GST_CAT_DEFAULT (rtpamrdepay_debug)
+
 /* references:
  *
  * RFC 3267 - Real-Time Transport Protocol (RTP) Payload Format and File
@@ -36,10 +39,10 @@
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_amrdepay_details =
-GST_ELEMENT_DETAILS ("RTP packet depayloader",
+GST_ELEMENT_DETAILS ("RTP AMR depayloader",
     "Codec/Depayloader/Network",
     "Extracts AMR or AMR-WB audio from RTP packets (RFC 3267)",
-    "Wim Taymans <wim@fluendo.com>");
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 /* RtpAMRDepay signals and args */
 enum
@@ -146,6 +149,9 @@ gst_rtp_amr_depay_class_init (GstRtpAMRDepayClass * klass)
 
   gstbasertpdepayload_class->process = gst_rtp_amr_depay_process;
   gstbasertpdepayload_class->set_caps = gst_rtp_amr_depay_setcaps;
+
+  GST_DEBUG_CATEGORY_INIT (rtpamrdepay_debug, "rtpamrdepay", 0,
+      "AMR/AMR-WB RTP Depayloader");
 }
 
 static void
@@ -168,6 +174,7 @@ gst_rtp_amr_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   const gchar *params;
   const gchar *str, *type;
   gint clock_rate, need_clock_rate;
+  gboolean res;
 
   rtpamrdepay = GST_RTP_AMR_DEPAY (depayload);
 
@@ -177,11 +184,11 @@ gst_rtp_amr_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   if ((str = gst_structure_get_string (structure, "encoding-name"))) {
     if (strcmp (str, "AMR") == 0) {
       rtpamrdepay->mode = GST_RTP_AMR_DP_MODE_NB;
-      clock_rate = need_clock_rate = 8000;
+      need_clock_rate = 8000;
       type = "audio/AMR";
     } else if (strcmp (str, "AMR-WB") == 0) {
       rtpamrdepay->mode = GST_RTP_AMR_DP_MODE_WB;
-      clock_rate = need_clock_rate = 16000;
+      need_clock_rate = 16000;
       type = "audio/AMR-WB";
     } else
       goto invalid_mode;
@@ -229,7 +236,8 @@ gst_rtp_amr_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
     rtpamrdepay->channels = atoi (params);
   }
 
-  gst_structure_get_int (structure, "clock-rate", &clock_rate);
+  if (!gst_structure_get_int (structure, "clock-rate", &clock_rate))
+    clock_rate = need_clock_rate;
   depayload->clock_rate = clock_rate;
 
   /* we require 1 channel, 8000 Hz, octet aligned, no CRC,
@@ -248,13 +256,10 @@ gst_rtp_amr_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   srccaps = gst_caps_new_simple (type,
       "channels", G_TYPE_INT, rtpamrdepay->channels,
       "rate", G_TYPE_INT, clock_rate, NULL);
-
-  gst_pad_set_caps (GST_BASE_RTP_DEPAYLOAD_SRCPAD (depayload), srccaps);
+  res = gst_pad_set_caps (GST_BASE_RTP_DEPAYLOAD_SRCPAD (depayload), srccaps);
   gst_caps_unref (srccaps);
 
-  rtpamrdepay->negotiated = TRUE;
-
-  return TRUE;
+  return res;
 
   /* ERRORS */
 invalid_mode:
@@ -284,12 +289,6 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 
   rtpamrdepay = GST_RTP_AMR_DEPAY (depayload);
 
-  if (!rtpamrdepay->negotiated)
-    goto not_negotiated;
-
-  if (!gst_rtp_buffer_validate (buf))
-    goto invalid_packet;
-
   /* setup frame size pointer */
   if (rtpamrdepay->mode == GST_RTP_AMR_DP_MODE_NB)
     frame_size = nb_frame_size;
@@ -304,7 +303,9 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     gint i, num_packets, num_nonempty_packets;
     gint amr_len;
     gint ILL, ILP;
+    gboolean marker;
 
+    marker = gst_rtp_buffer_get_marker (buf);
     payload_len = gst_rtp_buffer_get_payload_len (buf);
 
     /* need at least 2 bytes for the header */
@@ -412,27 +413,19 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     /* we can set the duration because each packet is 20 milliseconds */
     GST_BUFFER_DURATION (outbuf) = num_packets * 20 * GST_MSECOND;
 
-    gst_buffer_set_caps (outbuf,
-        GST_PAD_CAPS (GST_BASE_RTP_DEPAYLOAD_SRCPAD (depayload)));
+    if (marker) {
+      /* marker bit marks a discont buffer after a talkspurt. */
+      GST_DEBUG_OBJECT (depayload, "marker bit was set");
+      GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
+      marker = FALSE;
+    }
 
-    GST_DEBUG ("gst_rtp_amr_depay_chain: pushing buffer of size %d",
+    GST_DEBUG_OBJECT (depayload, "pushing buffer of size %d",
         GST_BUFFER_SIZE (outbuf));
   }
   return outbuf;
 
   /* ERRORS */
-invalid_packet:
-  {
-    GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
-        (NULL), ("AMR RTP packet did not validate"));
-    goto bad_packet;
-  }
-not_negotiated:
-  {
-    GST_ELEMENT_ERROR (rtpamrdepay, STREAM, NOT_IMPLEMENTED,
-        (NULL), ("not negotiated"));
-    return NULL;
-  }
 too_small:
   {
     GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
