@@ -21,7 +21,6 @@
 #  include "config.h"
 #endif
 
-#include <stdio.h>
 #include <string.h>
 
 #include <gst/rtp/gstrtpbuffer.h>
@@ -31,13 +30,11 @@ GST_DEBUG_CATEGORY_STATIC (rtph264depay_debug);
 #define GST_CAT_DEFAULT (rtph264depay_debug)
 
 #define DEFAULT_BYTE_STREAM	TRUE
-#define DEFAULT_ACCESS_UNIT	FALSE
 
 enum
 {
   PROP_0,
   PROP_BYTE_STREAM,
-  PROP_ACCESS_UNIT,
   PROP_LAST
 };
 
@@ -45,6 +42,13 @@ enum
 /* 3 zero bytes syncword */
 static const guint8 sync_bytes[] = { 0, 0, 0, 1 };
 
+/* elementfactory information */
+static const GstElementDetails gst_rtp_h264depay_details =
+GST_ELEMENT_DETAILS ("RTP H264 depayloader",
+    "Codec/Depayloader/Network",
+    "Extracts H264 video from RTP packets (RFC 3984)",
+    "Wim Taymans <wim.taymans@gmail.com>");
+
 static GstStaticPadTemplate gst_rtp_h264_depay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -95,23 +99,18 @@ static GstBuffer *gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload,
     GstBuffer * buf);
 static gboolean gst_rtp_h264_depay_setcaps (GstBaseRTPDepayload * filter,
     GstCaps * caps);
-static gboolean gst_rtp_h264_depay_handle_event (GstBaseRTPDepayload * depay,
-    GstEvent * event);
 
 static void
 gst_rtp_h264_depay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_h264_depay_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_h264_depay_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_h264_depay_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_h264_depay_sink_template));
 
-  gst_element_class_set_details_simple (element_class, "RTP H264 depayloader",
-      "Codec/Depayloader/Network/RTP",
-      "Extracts H264 video from RTP packets (RFC 3984)",
-      "Wim Taymans <wim.taymans@gmail.com>");
+  gst_element_class_set_details (element_class, &gst_rtp_h264depay_details);
 }
 
 static void
@@ -132,18 +131,13 @@ gst_rtp_h264_depay_class_init (GstRtpH264DepayClass * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_BYTE_STREAM,
       g_param_spec_boolean ("byte-stream", "Byte Stream",
-          "Generate byte stream format of NALU (deprecated; use caps)",
-          DEFAULT_BYTE_STREAM, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_ACCESS_UNIT,
-      g_param_spec_boolean ("access-unit", "Access Unit",
-          "Merge NALU into AU (picture) (deprecated; use caps)",
-          DEFAULT_ACCESS_UNIT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          "Generate byte stream format of NALU", DEFAULT_BYTE_STREAM,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state = gst_rtp_h264_depay_change_state;
 
   gstbasertpdepayload_class->process = gst_rtp_h264_depay_process;
   gstbasertpdepayload_class->set_caps = gst_rtp_h264_depay_setcaps;
-  gstbasertpdepayload_class->handle_event = gst_rtp_h264_depay_handle_event;
 
   GST_DEBUG_CATEGORY_INIT (rtph264depay_debug, "rtph264depay", 0,
       "H264 Video RTP Depayloader");
@@ -154,21 +148,7 @@ gst_rtp_h264_depay_init (GstRtpH264Depay * rtph264depay,
     GstRtpH264DepayClass * klass)
 {
   rtph264depay->adapter = gst_adapter_new ();
-  rtph264depay->picture_adapter = gst_adapter_new ();
   rtph264depay->byte_stream = DEFAULT_BYTE_STREAM;
-  rtph264depay->merge = DEFAULT_ACCESS_UNIT;
-}
-
-static void
-gst_rtp_h264_depay_reset (GstRtpH264Depay * rtph264depay)
-{
-  gst_adapter_clear (rtph264depay->adapter);
-  rtph264depay->wait_start = TRUE;
-  gst_adapter_clear (rtph264depay->picture_adapter);
-  rtph264depay->picture_start = FALSE;
-  rtph264depay->last_keyframe = FALSE;
-  rtph264depay->last_ts = 0;
-  rtph264depay->current_fu_type = 0;
 }
 
 static void
@@ -182,7 +162,6 @@ gst_rtp_h264_depay_finalize (GObject * object)
     gst_buffer_unref (rtph264depay->codec_data);
 
   g_object_unref (rtph264depay->adapter);
-  g_object_unref (rtph264depay->picture_adapter);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -199,9 +178,6 @@ gst_rtp_h264_depay_set_property (GObject * object, guint prop_id,
     case PROP_BYTE_STREAM:
       rtph264depay->byte_stream = g_value_get_boolean (value);
       break;
-    case PROP_ACCESS_UNIT:
-      rtph264depay->merge = g_value_get_boolean (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -220,72 +196,58 @@ gst_rtp_h264_depay_get_property (GObject * object, guint prop_id,
     case PROP_BYTE_STREAM:
       g_value_set_boolean (value, rtph264depay->byte_stream);
       break;
-    case PROP_ACCESS_UNIT:
-      g_value_set_boolean (value, rtph264depay->merge);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
-static void
-gst_rtp_h264_depay_negotiate (GstRtpH264Depay * rtph264depay)
+static const guint8 a2bin[256] = {
+  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
+  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
+  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 62, 64, 64, 64, 63,
+  52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 64, 64, 64, 64, 64, 64,
+  64, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
+  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 64, 64, 64, 64, 64,
+  64, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
+  41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 64, 64, 64, 64, 64,
+  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
+  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
+  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
+  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
+  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
+  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
+  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
+  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64
+};
+
+static guint
+decode_base64 (gchar * in, guint8 * out)
 {
-  GstCaps *caps;
-  gint byte_stream = -1;
-  gint merge = -1;
+  guint8 v1, v2;
+  guint len = 0;
 
-  caps =
-      gst_pad_get_allowed_caps (GST_BASE_RTP_DEPAYLOAD_SRCPAD (rtph264depay));
-
-  GST_DEBUG_OBJECT (rtph264depay, "allowed caps: %" GST_PTR_FORMAT, caps);
-
-  if (caps) {
-    if (gst_caps_get_size (caps) > 0) {
-      GstStructure *s = gst_caps_get_structure (caps, 0);
-      const gchar *str = NULL;
-
-      if ((str = gst_structure_get_string (s, "stream-format"))) {
-        if (strcmp (str, "avc") == 0) {
-          byte_stream = FALSE;
-        } else if (strcmp (str, "byte-stream") == 0) {
-          byte_stream = TRUE;
-        } else {
-          GST_DEBUG_OBJECT (rtph264depay, "unknown stream-format: %s", str);
-        }
-      }
-
-      if ((str = gst_structure_get_string (s, "alignment"))) {
-        if (strcmp (str, "au") == 0) {
-          merge = TRUE;
-        } else if (strcmp (str, "nal") == 0) {
-          merge = FALSE;
-        } else {
-          GST_DEBUG_OBJECT (rtph264depay, "unknown alignment: %s", str);
-        }
-      }
-    }
-    gst_caps_unref (caps);
+  v1 = a2bin[(gint) * in];
+  while (v1 <= 63) {
+    /* read 4 bytes, write 3 bytes, invalid base64 are zeroes */
+    v2 = a2bin[(gint) * ++in];
+    *out++ = (v1 << 2) | ((v2 & 0x3f) >> 4);
+    v1 = (v2 > 63 ? 64 : a2bin[(gint) * ++in]);
+    *out++ = (v2 << 4) | ((v1 & 0x3f) >> 2);
+    v2 = (v1 > 63 ? 64 : a2bin[(gint) * ++in]);
+    *out++ = (v1 << 6) | (v2 & 0x3f);
+    v1 = (v2 > 63 ? 64 : a2bin[(gint) * ++in]);
+    len += 3;
   }
+  /* move to '\0' */
+  while (*in != '\0')
+    in++;
 
-  if (byte_stream >= 0) {
-    GST_DEBUG_OBJECT (rtph264depay, "downstream requires byte-stream %d",
-        byte_stream);
-    if (rtph264depay->byte_stream != byte_stream) {
-      GST_WARNING_OBJECT (rtph264depay,
-          "overriding property setting based on caps");
-      rtph264depay->byte_stream = byte_stream;
-    }
-  }
-  if (merge >= 0) {
-    GST_DEBUG_OBJECT (rtph264depay, "downstream requires merge %d", merge);
-    if (rtph264depay->merge != merge) {
-      GST_WARNING_OBJECT (rtph264depay,
-          "overriding property setting based on caps");
-      rtph264depay->merge = merge;
-    }
-  }
+  /* subtract padding */
+  while (len > 0 && *--in == '=')
+    len--;
+
+  return len;
 }
 
 static gboolean
@@ -313,9 +275,6 @@ gst_rtp_h264_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   /* hex: AVCProfileIndication:8 | profile_compat:8 | AVCLevelIndication:8 */
   profile = gst_structure_get_string (structure, "profile-level-id");
 
-  /* negotiate with downstream w.r.t. output format and alignment */
-  gst_rtp_h264_depay_negotiate (rtph264depay);
-
   if (rtph264depay->byte_stream && ps != NULL) {
     /* for bytestream we only need the parameter sets but we don't error out
      * when they are not there, we assume they are in the stream. */
@@ -337,16 +296,10 @@ gst_rtp_h264_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
     b64 = GST_BUFFER_DATA (codec_data);
     total = 0;
     for (i = 0; params[i]; i++) {
-      guint save = 0;
-      gint state = 0;
-
-      GST_DEBUG_OBJECT (depayload, "decoding param %d (%s)", i, params[i]);
+      GST_DEBUG_OBJECT (depayload, "decoding param %d", i);
       memcpy (b64, sync_bytes, sizeof (sync_bytes));
       b64 += sizeof (sync_bytes);
-      len =
-          g_base64_decode_step (params[i], strlen (params[i]), b64, &state,
-          &save);
-      GST_DEBUG_OBJECT (depayload, "decoded %d bytes", len);
+      len = decode_base64 (params[i], b64);
       total += len + sizeof (sync_bytes);
       b64 += len;
     }
@@ -365,8 +318,9 @@ gst_rtp_h264_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
     guint len, num_sps, num_pps;
     gint i;
     guint8 *data;
+    guint32 profile_id;
 
-    if (ps == NULL)
+    if (ps == NULL || profile == NULL)
       goto incomplete_caps;
 
     params = g_strsplit (ps, ",", 0);
@@ -381,16 +335,12 @@ gst_rtp_h264_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
     /* start with 7 bytes header */
     len = 7;
     for (i = 0; params[i]; i++) {
-      gsize nal_len;
+      gint nal_len;
       guint8 *nalp;
-      guint save = 0;
-      gint state = 0;
 
       nal_len = strlen (params[i]);
       nalp = g_malloc (nal_len + 2);
-
-      nal_len =
-          g_base64_decode_step (params[i], nal_len, nalp + 2, &state, &save);
+      nal_len = decode_base64 (params[i], nalp + 2);
       nalp[0] = (nal_len >> 8) & 0xff;
       nalp[1] = nal_len & 0xff;
       len += nal_len + 2;
@@ -406,31 +356,16 @@ gst_rtp_h264_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
     }
     g_strfreev (params);
 
-    if (num_sps == 0 || (GST_READ_UINT16_BE (sps[0]) < 3) || num_pps == 0) {
-      g_strfreev ((gchar **) pps);
-      g_strfreev ((gchar **) sps);
-      goto incomplete_caps;
-    }
-
     codec_data = gst_buffer_new_and_alloc (len);
     data = GST_BUFFER_DATA (codec_data);
 
     /* 8 bits version == 1 */
     *data++ = 1;
-    if (profile) {
-      guint32 profile_id;
-
-      /* hex: AVCProfileIndication:8 | profile_compat:8 | AVCLevelIndication:8 */
-      sscanf (profile, "%6x", &profile_id);
-      *data++ = (profile_id >> 16) & 0xff;
-      *data++ = (profile_id >> 8) & 0xff;
-      *data++ = profile_id & 0xff;
-    } else {
-      /* extract from SPS */
-      *data++ = sps[0][3];
-      *data++ = sps[0][4];
-      *data++ = sps[0][5];
-    }
+    /* hex: AVCProfileIndication:8 | profile_compat:8 | AVCLevelIndication:8 */
+    sscanf (profile, "%6x", &profile_id);
+    *data++ = (profile_id >> 16) & 0xff;
+    *data++ = (profile_id >> 8) & 0xff;
+    *data++ = profile_id & 0xff;
     /* 6 bits reserved | 2 bits lengthSizeMinusOn */
     *data++ = 0xff;
     /* 3 bits reserved | 5 bits numOfSequenceParameterSets */
@@ -460,13 +395,8 @@ gst_rtp_h264_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 
     gst_caps_set_simple (srccaps,
         "codec_data", GST_TYPE_BUFFER, codec_data, NULL);
-    gst_buffer_unref (codec_data);
   }
 
-  gst_caps_set_simple (srccaps, "stream-format", G_TYPE_STRING,
-      rtph264depay->byte_stream ? "byte-stream" : "avc",
-      "alignment", G_TYPE_STRING, rtph264depay->merge ? "au" : "nal", NULL);
-
   res = gst_pad_set_caps (depayload->srcpad, srccaps);
   gst_caps_unref (srccaps);
 
@@ -481,174 +411,13 @@ incomplete_caps:
   }
 }
 
-static GstBuffer *
-gst_rtp_h264_complete_au (GstRtpH264Depay * rtph264depay,
-    GstClockTime * out_timestamp, gboolean * out_keyframe)
-{
-  guint outsize;
-  GstBuffer *outbuf;
-
-  /* we had a picture in the adapter and we completed it */
-  GST_DEBUG_OBJECT (rtph264depay, "taking completed AU");
-  outsize = gst_adapter_available (rtph264depay->picture_adapter);
-  outbuf = gst_adapter_take_buffer (rtph264depay->picture_adapter, outsize);
-
-  *out_timestamp = rtph264depay->last_ts;
-  *out_keyframe = rtph264depay->last_keyframe;
-
-  rtph264depay->last_keyframe = FALSE;
-  rtph264depay->picture_start = FALSE;
-
-  return outbuf;
-}
-
-/* SPS/PPS/IDR considered key, all others DELTA;
- * so downstream waiting for keyframe can pick up at SPS/PPS/IDR */
-#define NAL_TYPE_IS_KEY(nt) (((nt) == 5) || ((nt) == 7) || ((nt) == 8))
-
-static GstBuffer *
-gst_rtp_h264_depay_handle_nal (GstRtpH264Depay * rtph264depay, GstBuffer * nal,
-    GstClockTime in_timestamp, gboolean marker)
-{
-  GstBaseRTPDepayload *depayload = GST_BASE_RTP_DEPAYLOAD (rtph264depay);
-  gint nal_type;
-  guint size;
-  guint8 *data;
-  GstBuffer *outbuf = NULL;
-  GstClockTime out_timestamp;
-  gboolean keyframe, out_keyframe;
-
-  size = GST_BUFFER_SIZE (nal);
-  if (G_UNLIKELY (size < 5))
-    goto short_nal;
-
-  data = GST_BUFFER_DATA (nal);
-
-  nal_type = data[4] & 0x1f;
-  GST_DEBUG_OBJECT (rtph264depay, "handle NAL type %d", nal_type);
-
-  keyframe = NAL_TYPE_IS_KEY (nal_type);
-
-  out_keyframe = keyframe;
-  out_timestamp = in_timestamp;
-
-  if (rtph264depay->merge) {
-    gboolean start = FALSE, complete = FALSE;
-
-    /* consider a coded slices (IDR or not) to start a picture,
-     * (so ending the previous one) if first_mb_in_slice == 0
-     * (non-0 is part of previous one) */
-    /* NOTE this is not entirely according to Access Unit specs in 7.4.1.2.4,
-     * but in practice it works in sane cases, needs not much parsing,
-     * and also works with broken frame_num in NAL (where spec-wise would fail) */
-    if (nal_type == 1 || nal_type == 2 || nal_type == 5) {
-      /* we have a picture start */
-      start = TRUE;
-      if (data[5] & 0x80) {
-        /* first_mb_in_slice == 0 completes a picture */
-        complete = TRUE;
-      }
-    } else if (nal_type >= 6 && nal_type <= 9) {
-      /* SEI, SPS, PPS, AU terminate picture */
-      complete = TRUE;
-    }
-    GST_DEBUG_OBJECT (depayload, "start %d, complete %d", start, complete);
-
-    if (complete && rtph264depay->picture_start)
-      outbuf = gst_rtp_h264_complete_au (rtph264depay, &out_timestamp,
-          &out_keyframe);
-
-    /* add to adapter */
-    GST_DEBUG_OBJECT (depayload, "adding NAL to picture adapter");
-    gst_adapter_push (rtph264depay->picture_adapter, nal);
-    rtph264depay->last_ts = in_timestamp;
-    rtph264depay->last_keyframe |= keyframe;
-    rtph264depay->picture_start |= start;
-
-    if (marker)
-      outbuf = gst_rtp_h264_complete_au (rtph264depay, &out_timestamp,
-          &out_keyframe);
-  } else {
-    /* no merge, output is input nal */
-    GST_DEBUG_OBJECT (depayload, "using NAL as output");
-    outbuf = nal;
-  }
-
-  if (outbuf) {
-    /* prepend codec_data */
-    if (rtph264depay->codec_data) {
-      GST_DEBUG_OBJECT (depayload, "prepending codec_data");
-      outbuf = gst_buffer_join (rtph264depay->codec_data, outbuf);
-      rtph264depay->codec_data = NULL;
-      out_keyframe = TRUE;
-    }
-    outbuf = gst_buffer_make_metadata_writable (outbuf);
-
-    GST_BUFFER_TIMESTAMP (outbuf) = out_timestamp;
-
-    if (out_keyframe)
-      GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
-    else
-      GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
-
-    gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
-  }
-
-  return outbuf;
-
-  /* ERRORS */
-short_nal:
-  {
-    GST_WARNING_OBJECT (depayload, "dropping short NAL");
-    gst_buffer_unref (nal);
-    return NULL;
-  }
-}
-
-static GstBuffer *
-gst_rtp_h264_push_fragmentation_unit (GstRtpH264Depay * rtph264depay,
-    gboolean send)
-{
-  guint outsize;
-  guint8 *outdata;
-  GstBuffer *outbuf;
-
-  outsize = gst_adapter_available (rtph264depay->adapter);
-  outbuf = gst_adapter_take_buffer (rtph264depay->adapter, outsize);
-  outdata = GST_BUFFER_DATA (outbuf);
-
-  GST_DEBUG_OBJECT (rtph264depay, "output %d bytes", outsize);
-
-  if (rtph264depay->byte_stream) {
-    memcpy (outdata, sync_bytes, sizeof (sync_bytes));
-  } else {
-    outsize -= 4;
-    outdata[0] = (outsize >> 24);
-    outdata[1] = (outsize >> 16);
-    outdata[2] = (outsize >> 8);
-    outdata[3] = (outsize);
-  }
-
-  rtph264depay->current_fu_type = 0;
-
-  if (send) {
-    outbuf = gst_rtp_h264_depay_handle_nal (rtph264depay, outbuf,
-        rtph264depay->fu_timestamp, rtph264depay->fu_marker);
-    if (outbuf)
-      gst_base_rtp_depayload_push (GST_BASE_RTP_DEPAYLOAD (rtph264depay),
-          outbuf);
-    return NULL;
-  } else {
-    return gst_rtp_h264_depay_handle_nal (rtph264depay, outbuf,
-        rtph264depay->fu_timestamp, rtph264depay->fu_marker);
-  }
-}
-
+/* FIXME, non-bytestream handling is freaking out ffmpeg. Apparently we need to
+ * group all NAL units belonging to one frame together */
 static GstBuffer *
 gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 {
   GstRtpH264Depay *rtph264depay;
-  GstBuffer *outbuf = NULL;
+  GstBuffer *outbuf;
   guint8 nal_unit_type;
 
   rtph264depay = GST_RTP_H264_DEPAY (depayload);
@@ -657,7 +426,6 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   if (GST_BUFFER_IS_DISCONT (buf)) {
     gst_adapter_clear (rtph264depay->adapter);
     rtph264depay->wait_start = TRUE;
-    rtph264depay->current_fu_type = 0;
   }
 
   {
@@ -667,20 +435,12 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     guint8 nal_ref_idc;
     guint8 *outdata;
     guint outsize, nalu_size;
-    GstClockTime timestamp;
-    gboolean marker;
-
-    timestamp = GST_BUFFER_TIMESTAMP (buf);
 
     payload_len = gst_rtp_buffer_get_payload_len (buf);
     payload = gst_rtp_buffer_get_payload (buf);
-    marker = gst_rtp_buffer_get_marker (buf);
 
     GST_DEBUG_OBJECT (rtph264depay, "receiving %d bytes", payload_len);
 
-    if (payload_len == 0)
-      return NULL;
-
     /* +---------------+
      * |0|1|2|3|4|5|6|7|
      * +-+-+-+-+-+-+-+-+
@@ -698,13 +458,6 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     GST_DEBUG_OBJECT (rtph264depay, "NRI %d, Type %d", nal_ref_idc,
         nal_unit_type);
 
-    /* If FU unit was being processed, but the current nal is of a different
-     * type.  Assume that the remote payloader is buggy (didn't set the end bit
-     * when the FU ended) and send out what we gathered thusfar */
-    if (G_UNLIKELY (rtph264depay->current_fu_type != 0 &&
-            nal_unit_type != rtph264depay->current_fu_type))
-      gst_rtp_h264_push_fragmentation_unit (rtph264depay, TRUE);
-
     switch (nal_unit_type) {
       case 0:
       case 30:
@@ -724,6 +477,11 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 
         rtph264depay->wait_start = FALSE;
 
+        /* prepend codec_data */
+        if (rtph264depay->codec_data) {
+          gst_adapter_push (rtph264depay->adapter, rtph264depay->codec_data);
+          rtph264depay->codec_data = NULL;
+        }
 
         /* STAP-A    Single-time aggregation packet     5.7.1 */
         while (payload_len > 2) {
@@ -735,9 +493,8 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
            */
           nalu_size = (payload[0] << 8) | payload[1];
 
-          /* dont include nalu_size */
-          if (nalu_size > (payload_len - 2))
-            nalu_size = payload_len - 2;
+          if (nalu_size > payload_len)
+            nalu_size = payload_len;
 
           outsize = nalu_size + sizeof (sync_bytes);
           outbuf = gst_buffer_new_and_alloc (outsize);
@@ -766,9 +523,9 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
         outsize = gst_adapter_available (rtph264depay->adapter);
         outbuf = gst_adapter_take_buffer (rtph264depay->adapter, outsize);
 
-        outbuf = gst_rtp_h264_depay_handle_nal (rtph264depay, outbuf, timestamp,
-            marker);
-        break;
+        gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
+
+        return outbuf;
       }
       case 26:
         /* MTAP16    Multi-time aggregation packet      5.7.2 */
@@ -806,15 +563,6 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
           /* NAL unit starts here */
           guint8 nal_header;
 
-          /* If a new FU unit started, while still processing an older one.
-           * Assume that the remote payloader is buggy (doesn't set the end
-           * bit) and send out what we've gathered thusfar */
-          if (G_UNLIKELY (rtph264depay->current_fu_type != 0))
-            gst_rtp_h264_push_fragmentation_unit (rtph264depay, TRUE);
-
-          rtph264depay->current_fu_type = nal_unit_type;
-          rtph264depay->fu_timestamp = timestamp;
-
           rtph264depay->wait_start = FALSE;
 
           /* reconstruct NAL header */
@@ -853,12 +601,34 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
           gst_adapter_push (rtph264depay->adapter, outbuf);
         }
 
-        outbuf = NULL;
-        rtph264depay->fu_marker = marker;
-
         /* if NAL unit ends, flush the adapter */
-        if (E)
-          outbuf = gst_rtp_h264_push_fragmentation_unit (rtph264depay, FALSE);
+        if (E) {
+          GST_DEBUG_OBJECT (rtph264depay, "output %d bytes", outsize);
+
+          outsize = gst_adapter_available (rtph264depay->adapter);
+          outbuf = gst_adapter_take_buffer (rtph264depay->adapter, outsize);
+          outdata = GST_BUFFER_DATA (outbuf);
+
+          if (rtph264depay->byte_stream) {
+            memcpy (outdata, sync_bytes, sizeof (sync_bytes));
+          } else {
+            outsize -= 4;
+            outdata[0] = (outsize >> 24);
+            outdata[1] = (outsize >> 16);
+            outdata[2] = (outsize >> 8);
+            outdata[3] = (outsize);
+          }
+          gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
+
+          /* push codec_data first */
+          if (rtph264depay->codec_data) {
+            gst_buffer_set_caps (rtph264depay->codec_data,
+                GST_PAD_CAPS (depayload->srcpad));
+            gst_base_rtp_depayload_push (depayload, rtph264depay->codec_data);
+            rtph264depay->codec_data = NULL;
+          }
+          return outbuf;
+        }
         break;
       }
       default:
@@ -881,14 +651,21 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
         outdata += sizeof (sync_bytes);
         memcpy (outdata, payload, nalu_size);
 
-        outbuf = gst_rtp_h264_depay_handle_nal (rtph264depay, outbuf, timestamp,
-            marker);
-        break;
+        gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
+
+        /* push codec_data first */
+        if (rtph264depay->codec_data) {
+          gst_buffer_set_caps (rtph264depay->codec_data,
+              GST_PAD_CAPS (depayload->srcpad));
+          gst_base_rtp_depayload_push (depayload, rtph264depay->codec_data);
+          rtph264depay->codec_data = NULL;
+        }
+        return outbuf;
       }
     }
   }
 
-  return outbuf;
+  return NULL;
 
   /* ERRORS */
 undefined_type:
@@ -910,25 +687,6 @@ not_implemented:
   }
 }
 
-static gboolean
-gst_rtp_h264_depay_handle_event (GstBaseRTPDepayload * depay, GstEvent * event)
-{
-  GstRtpH264Depay *rtph264depay;
-
-  rtph264depay = GST_RTP_H264_DEPAY (depay);
-
-  switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_FLUSH_STOP:
-      gst_rtp_h264_depay_reset (rtph264depay);
-      break;
-    default:
-      break;
-  }
-
-  return
-      GST_BASE_RTP_DEPAYLOAD_CLASS (parent_class)->handle_event (depay, event);
-}
-
 static GstStateChangeReturn
 gst_rtp_h264_depay_change_state (GstElement * element,
     GstStateChange transition)
@@ -942,7 +700,8 @@ gst_rtp_h264_depay_change_state (GstElement * element,
     case GST_STATE_CHANGE_NULL_TO_READY:
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      gst_rtp_h264_depay_reset (rtph264depay);
+      gst_adapter_clear (rtph264depay->adapter);
+      rtph264depay->wait_start = TRUE;
       break;
     default:
       break;
@@ -963,5 +722,5 @@ gboolean
 gst_rtp_h264_depay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtph264depay",
-      GST_RANK_SECONDARY, GST_TYPE_RTP_H264_DEPAY);
+      GST_RANK_MARGINAL, GST_TYPE_RTP_H264_DEPAY);
 }
