@@ -23,15 +23,19 @@
 
 #include <gst/rtp/gstrtpbuffer.h>
 
-#include <math.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include "gstrtpjpegdepay.h"
 
 GST_DEBUG_CATEGORY_STATIC (rtpjpegdepay_debug);
 #define GST_CAT_DEFAULT (rtpjpegdepay_debug)
 
+/* elementfactory information */
+static const GstElementDetails gst_rtp_jpegdepay_details =
+GST_ELEMENT_DETAILS ("RTP JPEG depayloader",
+    "Codec/Depayloader/Network",
+    "Extracts JPEG video from RTP packets (RFC 2435)",
+    "Wim Taymans <wim.taymans@gmail.com>");
+
 static GstStaticPadTemplate gst_rtp_jpeg_depay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -46,24 +50,11 @@ static GstStaticPadTemplate gst_rtp_jpeg_depay_sink_template =
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"video\", "
         "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
-        "clock-rate = (int) 90000, " "encoding-name = (string) \"JPEG\"; "
-        /* optional SDP attributes */
-        /*
-         * "a-framerate = (string) 0.00, "
-         * "x-framerate = (string) 0.00, "
-         * "x-dimensions = (string) \"1234,1234\", "
-         */
+        "clock-rate = (int) 90000, " "encoding-name = (string) \"JPEG\";"
         "application/x-rtp, "
         "media = (string) \"video\", "
         "payload = (int) " GST_RTP_PAYLOAD_JPEG_STRING ", "
-        "clock-rate = (int) 90000"
-        /* optional SDP attributes */
-        /*
-         * "a-framerate = (string) 0.00, "
-         * "x-framerate = (string) 0.00, "
-         * "x-dimensions = (string) \"1234,1234\""
-         */
-    )
+        "clock-rate = (int) 90000")
     );
 
 GST_BOILERPLATE (GstRtpJPEGDepay, gst_rtp_jpeg_depay, GstBaseRTPDepayload,
@@ -71,28 +62,25 @@ GST_BOILERPLATE (GstRtpJPEGDepay, gst_rtp_jpeg_depay, GstBaseRTPDepayload,
 
 static void gst_rtp_jpeg_depay_finalize (GObject * object);
 
-static GstStateChangeReturn gst_rtp_jpeg_depay_change_state (GstElement *
-    element, GstStateChange transition);
-
 static gboolean gst_rtp_jpeg_depay_setcaps (GstBaseRTPDepayload * depayload,
     GstCaps * caps);
 static GstBuffer *gst_rtp_jpeg_depay_process (GstBaseRTPDepayload * depayload,
     GstBuffer * buf);
 
+static GstStateChangeReturn gst_rtp_jpeg_depay_change_state (GstElement *
+    element, GstStateChange transition);
+
 static void
 gst_rtp_jpeg_depay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_jpeg_depay_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_jpeg_depay_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_jpeg_depay_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_jpeg_depay_sink_template));
 
-  gst_element_class_set_details_simple (element_class, "RTP JPEG depayloader",
-      "Codec/Depayloader/Network/RTP",
-      "Extracts JPEG video from RTP packets (RFC 2435)",
-      "Wim Taymans <wim.taymans@gmail.com>");
+  gst_element_class_set_details (element_class, &gst_rtp_jpegdepay_details);
 }
 
 static void
@@ -108,6 +96,8 @@ gst_rtp_jpeg_depay_class_init (GstRtpJPEGDepayClass * klass)
 
   gobject_class->finalize = gst_rtp_jpeg_depay_finalize;
 
+  parent_class = g_type_class_peek_parent (klass);
+
   gstelement_class->change_state = gst_rtp_jpeg_depay_change_state;
 
   gstbasertpdepayload_class->set_caps = gst_rtp_jpeg_depay_setcaps;
@@ -124,27 +114,6 @@ gst_rtp_jpeg_depay_init (GstRtpJPEGDepay * rtpjpegdepay,
   rtpjpegdepay->adapter = gst_adapter_new ();
 }
 
-static void
-gst_rtp_jpeg_depay_reset (GstRtpJPEGDepay * depay)
-{
-  gint i;
-
-  depay->width = 0;
-  depay->height = 0;
-  depay->media_width = 0;
-  depay->media_height = 0;
-  depay->frate_num = 0;
-  depay->frate_denom = 1;
-  depay->discont = TRUE;
-
-  for (i = 0; i < 255; i++) {
-    g_free (depay->qtables[i]);
-    depay->qtables[i] = NULL;
-  }
-
-  gst_adapter_clear (depay->adapter);
-}
-
 static void
 gst_rtp_jpeg_depay_finalize (GObject * object)
 {
@@ -152,8 +121,6 @@ gst_rtp_jpeg_depay_finalize (GObject * object)
 
   rtpjpegdepay = GST_RTP_JPEG_DEPAY (object);
 
-  gst_rtp_jpeg_depay_reset (rtpjpegdepay);
-
   g_object_unref (rtpjpegdepay->adapter);
   rtpjpegdepay->adapter = NULL;
 
@@ -378,7 +345,7 @@ MakeHeaders (guint8 * p, int type, int width, int height, guint8 * qt,
   *p++ = width;                 /* width lsb */
   *p++ = 3;                     /* number of components */
   *p++ = 0;                     /* comp 0 */
-  if ((type & 0x3f) == 0)
+  if (type == 0)
     *p++ = 0x21;                /* hsamp = 2, vsamp = 1 */
   else
     *p++ = 0x22;                /* hsamp = 2, vsamp = 2 */
@@ -420,66 +387,27 @@ MakeHeaders (guint8 * p, int type, int width, int height, guint8 * qt,
 static gboolean
 gst_rtp_jpeg_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 {
-  GstRtpJPEGDepay *rtpjpegdepay;
   GstStructure *structure;
+  GstRtpJPEGDepay *rtpjpegdepay;
+  GstCaps *outcaps;
   gint clock_rate;
-  const gchar *media_attr;
+  gboolean res;
 
   rtpjpegdepay = GST_RTP_JPEG_DEPAY (depayload);
 
   structure = gst_caps_get_structure (caps, 0);
-  GST_DEBUG_OBJECT (rtpjpegdepay, "Caps set: %" GST_PTR_FORMAT, caps);
 
   if (!gst_structure_get_int (structure, "clock-rate", &clock_rate))
     clock_rate = 90000;
   depayload->clock_rate = clock_rate;
 
-  /* reset defaults */
-  rtpjpegdepay->width = 0;
-  rtpjpegdepay->height = 0;
-  rtpjpegdepay->media_width = 0;
-  rtpjpegdepay->media_height = 0;
-  rtpjpegdepay->frate_num = 0;
-  rtpjpegdepay->frate_denom = 1;
+  outcaps =
+      gst_caps_new_simple ("image/jpeg", "framerate", GST_TYPE_FRACTION, 0, 1,
+      NULL);
+  res = gst_pad_set_caps (depayload->srcpad, outcaps);
+  gst_caps_unref (outcaps);
 
-  /* check for optional SDP attributes */
-  if ((media_attr = gst_structure_get_string (structure, "x-dimensions"))) {
-    gint w, h;
-
-    if (sscanf (media_attr, "%d,%d", &w, &h) == 2) {
-      rtpjpegdepay->media_width = w;
-      rtpjpegdepay->media_height = h;
-    }
-  }
-
-  /* try to get a framerate */
-  media_attr = gst_structure_get_string (structure, "a-framerate");
-  if (!media_attr)
-    media_attr = gst_structure_get_string (structure, "x-framerate");
-
-  if (media_attr) {
-    GValue src = { 0 };
-    GValue dest = { 0 };
-    gchar *s;
-
-    /* canonicalise floating point string so we can handle framerate strings
-     * in the form "24.930" or "24,930" irrespective of the current locale */
-    s = g_strdup (media_attr);
-    g_strdelimit (s, ",", '.');
-
-    /* convert the float to a fraction */
-    g_value_init (&src, G_TYPE_DOUBLE);
-    g_value_set_double (&src, g_ascii_strtod (s, NULL));
-    g_value_init (&dest, GST_TYPE_FRACTION);
-    g_value_transform (&src, &dest);
-
-    rtpjpegdepay->frate_num = gst_value_get_fraction_numerator (&dest);
-    rtpjpegdepay->frate_denom = gst_value_get_fraction_denominator (&dest);
-
-    g_free (s);
-  }
-
-  return TRUE;
+  return res;
 }
 
 static GstBuffer *
@@ -499,7 +427,6 @@ gst_rtp_jpeg_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 
   if (GST_BUFFER_IS_DISCONT (buf)) {
     gst_adapter_clear (rtpjpegdepay->adapter);
-    rtpjpegdepay->discont = TRUE;
   }
 
   payload_len = gst_rtp_buffer_get_payload_len (buf);
@@ -524,17 +451,6 @@ gst_rtp_jpeg_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   width = payload[6] * 8;
   height = payload[7] * 8;
 
-  /* allow frame dimensions > 2040, passed in SDP session or media attributes
-   * from gstrtspsrc.c (gst_rtspsrc_sdp_attributes_to_caps), or in caps */
-  if (!width)
-    width = rtpjpegdepay->media_width;
-
-  if (!height)
-    height = rtpjpegdepay->media_height;
-
-  if (width == 0 || height == 0)
-    goto invalid_dimension;
-
   GST_DEBUG_OBJECT (rtpjpegdepay, "frag %u, type %u, Q %d, width %u, height %u",
       frag_offset, type, Q, width, height);
 
@@ -591,10 +507,7 @@ gst_rtp_jpeg_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     if (length > payload_len)
       goto empty_packet;
 
-    if (length > 0)
-      qtable = payload;
-    else
-      qtable = rtpjpegdepay->qtables[Q];
+    qtable = payload;
 
     payload += length;
     header_len += length;
@@ -608,23 +521,6 @@ gst_rtp_jpeg_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   if (frag_offset == 0) {
     guint size;
 
-    if (rtpjpegdepay->width != width || rtpjpegdepay->height != height) {
-      GstCaps *outcaps;
-
-      outcaps =
-          gst_caps_new_simple ("image/jpeg", "framerate", GST_TYPE_FRACTION,
-          rtpjpegdepay->frate_num, rtpjpegdepay->frate_denom, "width",
-          G_TYPE_INT, width, "height", G_TYPE_INT, height, NULL);
-      gst_pad_set_caps (depayload->srcpad, outcaps);
-      gst_caps_unref (outcaps);
-
-      rtpjpegdepay->width = width;
-      rtpjpegdepay->height = height;
-    }
-
-    GST_LOG_OBJECT (rtpjpegdepay, "first packet, length %" G_GUINT16_FORMAT,
-        length);
-
     /* first packet */
     if (length == 0) {
       if (Q < 128) {
@@ -641,9 +537,6 @@ gst_rtp_jpeg_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
         }
         /* all 8 bit quantizers */
         precision = 0;
-      } else {
-        if (!qtable)
-          goto no_qtable;
       }
     }
     /* max header length, should be big enough */
@@ -651,7 +544,7 @@ gst_rtp_jpeg_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     size = MakeHeaders (GST_BUFFER_DATA (outbuf), type,
         width, height, qtable, precision, dri);
 
-    GST_DEBUG_OBJECT (rtpjpegdepay, "pushing %u bytes of header", size);
+    GST_DEBUG_OBJECT (rtpjpegdepay, "pushing %u bytes header", size);
 
     GST_BUFFER_SIZE (outbuf) = size;
 
@@ -691,11 +584,6 @@ gst_rtp_jpeg_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     }
     outbuf = gst_adapter_take_buffer (rtpjpegdepay->adapter, avail);
 
-    if (rtpjpegdepay->discont) {
-      GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
-      rtpjpegdepay->discont = FALSE;
-    }
-
     GST_DEBUG_OBJECT (rtpjpegdepay, "returning %u bytes", avail);
   }
 
@@ -708,20 +596,8 @@ empty_packet:
         ("Empty Payload."), (NULL));
     return NULL;
   }
-invalid_dimension:
-  {
-    GST_ELEMENT_WARNING (rtpjpegdepay, STREAM, FORMAT,
-        ("Invalid Dimension %dx%d.", width, height), (NULL));
-    return NULL;
-  }
-no_qtable:
-  {
-    GST_WARNING_OBJECT (rtpjpegdepay, "no qtable");
-    return NULL;
-  }
 }
 
-
 static GstStateChangeReturn
 gst_rtp_jpeg_depay_change_state (GstElement * element,
     GstStateChange transition)
@@ -732,8 +608,7 @@ gst_rtp_jpeg_depay_change_state (GstElement * element,
   rtpjpegdepay = GST_RTP_JPEG_DEPAY (element);
 
   switch (transition) {
-    case GST_STATE_CHANGE_READY_TO_PAUSED:
-      gst_rtp_jpeg_depay_reset (rtpjpegdepay);
+    case GST_STATE_CHANGE_NULL_TO_READY:
       break;
     default:
       break;
@@ -742,7 +617,7 @@ gst_rtp_jpeg_depay_change_state (GstElement * element,
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
-    case GST_STATE_CHANGE_PAUSED_TO_READY:
+    case GST_STATE_CHANGE_READY_TO_NULL:
       break;
     default:
       break;
@@ -750,10 +625,9 @@ gst_rtp_jpeg_depay_change_state (GstElement * element,
   return ret;
 }
 
-
 gboolean
 gst_rtp_jpeg_depay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpjpegdepay",
-      GST_RANK_SECONDARY, GST_TYPE_RTP_JPEG_DEPAY);
+      GST_RANK_MARGINAL, GST_TYPE_RTP_JPEG_DEPAY);
 }
