@@ -30,14 +30,20 @@
 GST_DEBUG_CATEGORY_STATIC (rtpmp4gdepay_debug);
 #define GST_CAT_DEFAULT (rtpmp4gdepay_debug)
 
+/* elementfactory information */
+static const GstElementDetails gst_rtp_mp4gdepay_details =
+GST_ELEMENT_DETAILS ("RTP MPEG4 ES depayloader",
+    "Codec/Depayloader/Network",
+    "Extracts MPEG4 elementary streams from RTP packets (RFC 3640)",
+    "Wim Taymans <wim.taymans@gmail.com>");
+
 static GstStaticPadTemplate gst_rtp_mp4g_depay_src_template =
     GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/mpeg,"
         "mpegversion=(int) 4,"
-        "systemstream=(boolean)false;"
-        "audio/mpeg," "mpegversion=(int) 4, " "stream-format=(string)raw")
+        "systemstream=(boolean)false;" "audio/mpeg," "mpegversion=(int) 4")
     );
 
 static GstStaticPadTemplate gst_rtp_mp4g_depay_sink_template =
@@ -135,8 +141,6 @@ static gboolean gst_rtp_mp4g_depay_setcaps (GstBaseRTPDepayload * depayload,
     GstCaps * caps);
 static GstBuffer *gst_rtp_mp4g_depay_process (GstBaseRTPDepayload * depayload,
     GstBuffer * buf);
-static gboolean gst_rtp_mp4g_depay_handle_event (GstBaseRTPDepayload * filter,
-    GstEvent * event);
 
 static GstStateChangeReturn gst_rtp_mp4g_depay_change_state (GstElement *
     element, GstStateChange transition);
@@ -147,15 +151,12 @@ gst_rtp_mp4g_depay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_mp4g_depay_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_mp4g_depay_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_mp4g_depay_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_mp4g_depay_sink_template));
 
-  gst_element_class_set_details_simple (element_class,
-      "RTP MPEG4 ES depayloader", "Codec/Depayloader/Network/RTP",
-      "Extracts MPEG4 elementary streams from RTP packets (RFC 3640)",
-      "Wim Taymans <wim.taymans@gmail.com>");
+  gst_element_class_set_details (element_class, &gst_rtp_mp4gdepay_details);
 }
 
 static void
@@ -169,13 +170,14 @@ gst_rtp_mp4g_depay_class_init (GstRtpMP4GDepayClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
+  parent_class = g_type_class_peek_parent (klass);
+
   gobject_class->finalize = gst_rtp_mp4g_depay_finalize;
 
   gstelement_class->change_state = gst_rtp_mp4g_depay_change_state;
 
   gstbasertpdepayload_class->process = gst_rtp_mp4g_depay_process;
   gstbasertpdepayload_class->set_caps = gst_rtp_mp4g_depay_setcaps;
-  gstbasertpdepayload_class->handle_event = gst_rtp_mp4g_depay_handle_event;
 
   GST_DEBUG_CATEGORY_INIT (rtpmp4gdepay_debug, "rtpmp4gdepay", 0,
       "MP4-generic RTP Depayloader");
@@ -242,12 +244,15 @@ gst_rtp_mp4g_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   if ((str = gst_structure_get_string (structure, "media"))) {
     if (strcmp (str, "audio") == 0) {
       srccaps = gst_caps_new_simple ("audio/mpeg",
-          "mpegversion", G_TYPE_INT, 4, "stream-format", G_TYPE_STRING, "raw",
-          NULL);
+          "mpegversion", G_TYPE_INT, 4, NULL);
+      /* AAC always has a default constant duration of 1024 but it can be
+       * overriden below. */
+      rtpmp4gdepay->constantDuration = 1024;
     } else if (strcmp (str, "video") == 0) {
       srccaps = gst_caps_new_simple ("video/mpeg",
           "mpegversion", G_TYPE_INT, 4,
           "systemstream", G_TYPE_BOOLEAN, FALSE, NULL);
+      rtpmp4gdepay->constantDuration = 0;
     }
   }
   if (srccaps == NULL)
@@ -274,9 +279,8 @@ gst_rtp_mp4g_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   rtpmp4gdepay->constantSize =
       gst_rtp_mp4g_depay_parse_int (structure, "constantsize", 0);
   rtpmp4gdepay->constantDuration =
-      gst_rtp_mp4g_depay_parse_int (structure, "constantduration", 0);
-  rtpmp4gdepay->maxDisplacement =
-      gst_rtp_mp4g_depay_parse_int (structure, "maxdisplacement", 0);
+      gst_rtp_mp4g_depay_parse_int (structure, "constantduration",
+      rtpmp4gdepay->constantDuration);
 
 
   /* get config string */
@@ -318,18 +322,6 @@ gst_rtp_mp4g_depay_clear_queue (GstRtpMP4GDepay * rtpmp4gdepay)
     gst_buffer_unref (outbuf);
 }
 
-static void
-gst_rtp_mp4g_depay_reset (GstRtpMP4GDepay * rtpmp4gdepay)
-{
-  gst_adapter_clear (rtpmp4gdepay->adapter);
-  rtpmp4gdepay->max_AU_index = -1;
-  rtpmp4gdepay->next_AU_index = -1;
-  rtpmp4gdepay->prev_AU_index = -1;
-  rtpmp4gdepay->prev_rtptime = -1;
-  rtpmp4gdepay->last_AU_index = -1;
-  gst_rtp_mp4g_depay_clear_queue (rtpmp4gdepay);
-}
-
 static void
 gst_rtp_mp4g_depay_flush_queue (GstRtpMP4GDepay * rtpmp4gdepay)
 {
@@ -452,8 +444,6 @@ gst_rtp_mp4g_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     payload_len = gst_rtp_buffer_get_payload_len (buf);
     payload = gst_rtp_buffer_get_payload (buf);
 
-    GST_DEBUG_OBJECT (rtpmp4gdepay, "received payload of %d", payload_len);
-
     rtptime = gst_rtp_buffer_get_timestamp (buf);
     M = gst_rtp_buffer_get_marker (buf);
 
@@ -490,23 +480,6 @@ gst_rtp_mp4g_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
       payload_AU = 2 + AU_headers_bytes;
       payload_AU_size = payload_len - AU_headers_bytes;
 
-      if (G_UNLIKELY (rtpmp4gdepay->auxiliarydatasizelength)) {
-        gint aux_size;
-
-        /* point the bitstream parser to the first auxiliary data bit */
-        gst_bs_parse_init (&bs, payload + AU_headers_bytes,
-            payload_len - AU_headers_bytes);
-        aux_size =
-            gst_bs_parse_read (&bs, rtpmp4gdepay->auxiliarydatasizelength);
-        /* convert to bytes */
-        aux_size = (aux_size + 7) / 8;
-        /* AU data then follows auxiliary data */
-        if (payload_AU_size < aux_size)
-          goto short_payload;
-        payload_AU += aux_size;
-        payload_AU_size -= aux_size;
-      }
-
       /* point the bitstream parser to the first AU header bit */
       gst_bs_parse_init (&bs, payload, payload_len);
       AU_index = AU_index_delta = 0;
@@ -538,12 +511,8 @@ gst_rtp_mp4g_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
          * reconstruct the AU ordering when interleaving. */
         if (i == 0) {
           AU_index = gst_bs_parse_read (&bs, rtpmp4gdepay->indexlength);
-
-          GST_DEBUG_OBJECT (rtpmp4gdepay, "AU index %u", AU_index);
-
           if (AU_index == 0 && rtpmp4gdepay->prev_AU_index == 0) {
             gint diff;
-            gint cd;
 
             /* if we see two consecutive packets with AU_index of 0, we can
              * assume we have constantDuration packets. Since we don't have
@@ -555,35 +524,14 @@ gst_rtp_mp4g_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
             else
               diff = -(rtpmp4gdepay->prev_rtptime - rtptime);
 
-            /* if no constantDuration was given, make one */
-            if (rtpmp4gdepay->constantDuration != 0) {
-              cd = rtpmp4gdepay->constantDuration;
-              GST_DEBUG_OBJECT (depayload, "using constantDuration %d", cd);
-            } else if (rtpmp4gdepay->prev_AU_num > 0) {
-              /* use number of packets and of previous frame */
-              cd = diff / rtpmp4gdepay->prev_AU_num;
-              GST_DEBUG_OBJECT (depayload, "guessing constantDuration %d", cd);
-            } else {
-              /* assume this frame has the same number of packets as the
-               * previous one */
-              cd = diff / num_AU_headers;
-              GST_DEBUG_OBJECT (depayload, "guessing constantDuration %d", cd);
-            }
-
-            if (cd > 0) {
-              /* get the number of packets by dividing with the duration */
-              diff /= cd;
-            } else {
-              diff = 0;
-            }
+            /* get the number of packets by dividing with the duration */
+            diff /= rtpmp4gdepay->constantDuration;
 
             rtpmp4gdepay->last_AU_index += diff;
             rtpmp4gdepay->prev_AU_index = AU_index;
 
             AU_index = rtpmp4gdepay->last_AU_index;
 
-            GST_DEBUG_OBJECT (rtpmp4gdepay, "diff %d, AU index %u", diff,
-                AU_index);
           } else {
             rtpmp4gdepay->prev_AU_index = AU_index;
             rtpmp4gdepay->last_AU_index = AU_index;
@@ -596,26 +544,7 @@ gst_rtp_mp4g_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
             /* a new interleave group started, flush */
             gst_rtp_mp4g_depay_flush_queue (rtpmp4gdepay);
           }
-          if (G_UNLIKELY (!rtpmp4gdepay->maxDisplacement &&
-                  rtpmp4gdepay->max_AU_index != -1
-                  && rtpmp4gdepay->max_AU_index >= AU_index)) {
-            GstBuffer *outbuf;
-
-            /* some broken non-interleaved streams have AU-index jumping around
-             * all over the place, apparently assuming receiver disregards */
-            GST_DEBUG_OBJECT (rtpmp4gdepay, "non-interleaved broken AU indices;"
-                " forcing continuous flush");
-            /* reset AU to avoid repeated DISCONT in such case */
-            outbuf = g_queue_peek_head (rtpmp4gdepay->packets);
-            if (G_LIKELY (outbuf)) {
-              rtpmp4gdepay->next_AU_index = GST_BUFFER_OFFSET (outbuf);
-              gst_rtp_mp4g_depay_flush_queue (rtpmp4gdepay);
-            }
-            /* rebase next_AU_index to current rtp's first AU_index */
-            rtpmp4gdepay->next_AU_index = AU_index;
-          }
           rtpmp4gdepay->prev_rtptime = rtptime;
-          rtpmp4gdepay->prev_AU_num = num_AU_headers;
         } else {
           AU_index_delta =
               gst_bs_parse_read (&bs, rtpmp4gdepay->indexdeltalength);
@@ -716,28 +645,6 @@ short_payload:
   }
 }
 
-static gboolean
-gst_rtp_mp4g_depay_handle_event (GstBaseRTPDepayload * filter, GstEvent * event)
-{
-  gboolean ret;
-  GstRtpMP4GDepay *rtpmp4gdepay;
-
-  rtpmp4gdepay = GST_RTP_MP4G_DEPAY (filter);
-
-  switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_FLUSH_STOP:
-      gst_rtp_mp4g_depay_reset (rtpmp4gdepay);
-      break;
-    default:
-      break;
-  }
-
-  ret =
-      GST_BASE_RTP_DEPAYLOAD_CLASS (parent_class)->handle_event (filter, event);
-
-  return ret;
-}
-
 static GstStateChangeReturn
 gst_rtp_mp4g_depay_change_state (GstElement * element,
     GstStateChange transition)
@@ -749,7 +656,12 @@ gst_rtp_mp4g_depay_change_state (GstElement * element,
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      gst_rtp_mp4g_depay_reset (rtpmp4gdepay);
+      gst_adapter_clear (rtpmp4gdepay->adapter);
+      rtpmp4gdepay->max_AU_index = -1;
+      rtpmp4gdepay->next_AU_index = -1;
+      rtpmp4gdepay->prev_AU_index = -1;
+      rtpmp4gdepay->prev_rtptime = -1;
+      rtpmp4gdepay->last_AU_index = -1;
       break;
     default:
       break;
@@ -759,7 +671,8 @@ gst_rtp_mp4g_depay_change_state (GstElement * element,
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      gst_rtp_mp4g_depay_reset (rtpmp4gdepay);
+      gst_adapter_clear (rtpmp4gdepay->adapter);
+      gst_rtp_mp4g_depay_clear_queue (rtpmp4gdepay);
       break;
     default:
       break;
@@ -771,5 +684,5 @@ gboolean
 gst_rtp_mp4g_depay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpmp4gdepay",
-      GST_RANK_SECONDARY, GST_TYPE_RTP_MP4G_DEPAY);
+      GST_RANK_MARGINAL, GST_TYPE_RTP_MP4G_DEPAY);
 }
