@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2005> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -32,10 +32,10 @@ GST_DEBUG_CATEGORY_STATIC (rtpmp4vpay_debug);
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_mp4vpay_details =
-GST_ELEMENT_DETAILS ("RTP MPEG-4 Video packet payloader",
+GST_ELEMENT_DETAILS ("RTP MPEG4 Video payloader",
     "Codec/Payloader/Network",
     "Payload MPEG-4 video as RTP packets (RFC 3016)",
-    "Wim Taymans <wim@fluendo.com>");
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 static GstStaticPadTemplate gst_rtp_mp4v_pay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -192,18 +192,19 @@ gst_rtp_mp4v_pay_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static void
+static gboolean
 gst_rtp_mp4v_pay_new_caps (GstRtpMP4VPay * rtpmp4vpay)
 {
   gchar *profile, *config;
   GValue v = { 0 };
+  gboolean res;
 
   profile = g_strdup_printf ("%d", rtpmp4vpay->profile);
   g_value_init (&v, GST_TYPE_BUFFER);
   gst_value_set_buffer (&v, rtpmp4vpay->config);
   config = gst_value_serialize (&v);
 
-  gst_basertppayload_set_outcaps (GST_BASE_RTP_PAYLOAD (rtpmp4vpay),
+  res = gst_basertppayload_set_outcaps (GST_BASE_RTP_PAYLOAD (rtpmp4vpay),
       "profile-level-id", G_TYPE_STRING, profile,
       "config", G_TYPE_STRING, config, NULL);
 
@@ -211,6 +212,8 @@ gst_rtp_mp4v_pay_new_caps (GstRtpMP4VPay * rtpmp4vpay)
 
   g_free (profile);
   g_free (config);
+
+  return res;
 }
 
 static gboolean
@@ -219,12 +222,15 @@ gst_rtp_mp4v_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
   GstRtpMP4VPay *rtpmp4vpay;
   GstStructure *structure;
   const GValue *codec_data;
+  gboolean res;
 
   rtpmp4vpay = GST_RTP_MP4V_PAY (payload);
 
   gst_basertppayload_set_options (payload, "video", TRUE, "MP4V-ES",
       rtpmp4vpay->rate);
 
+  res = TRUE;
+
   structure = gst_caps_get_structure (caps, 0);
   codec_data = gst_structure_get_value (structure, "codec_data");
   if (codec_data) {
@@ -249,12 +255,12 @@ gst_rtp_mp4v_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
       if (rtpmp4vpay->config)
         gst_buffer_unref (rtpmp4vpay->config);
       rtpmp4vpay->config = gst_buffer_copy (buffer);
-      gst_rtp_mp4v_pay_new_caps (rtpmp4vpay);
+      res = gst_rtp_mp4v_pay_new_caps (rtpmp4vpay);
     }
   }
 
 done:
-  return TRUE;
+  return res;
 }
 
 static void
@@ -277,6 +283,12 @@ gst_rtp_mp4v_pay_flush (GstRtpMP4VPay * rtpmp4vpay)
    * over multiple packets. */
   avail = gst_adapter_available (rtpmp4vpay->adapter);
 
+  if (rtpmp4vpay->config == NULL) {
+    /* when we don't have a config yet, flush things out */
+    gst_adapter_flush (rtpmp4vpay->adapter, avail);
+    avail = 0;
+  }
+
   ret = GST_FLOW_OK;
 
   while (avail > 0) {
@@ -489,6 +501,9 @@ gst_rtp_mp4v_pay_event (GstPad * pad, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_NEWSEGMENT:
+    case GST_EVENT_EOS:
+      /* This flush call makes sure that the last buffer is always pushed
+       * to the base payloader */
       gst_rtp_mp4v_pay_flush (rtpmp4vpay);
       break;
     case GST_EVENT_FLUSH_STOP:
