@@ -90,10 +90,16 @@ gst_rtp_pcmu_depay_base_init (gpointer klass)
 static void
 gst_rtp_pcmu_depay_class_init (GstRtpPcmuDepayClass * klass)
 {
+  GObjectClass *gobject_class;
+  GstElementClass *gstelement_class;
   GstBaseRTPDepayloadClass *gstbasertpdepayload_class;
 
+  gobject_class = (GObjectClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
+  parent_class = g_type_class_peek_parent (klass);
+
   gstbasertpdepayload_class->process = gst_rtp_pcmu_depay_process;
   gstbasertpdepayload_class->set_caps = gst_rtp_pcmu_depay_setcaps;
 }
