@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2006> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2006> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -32,15 +32,15 @@ GST_DEBUG_CATEGORY_STATIC (rtpvorbispay_debug);
 #define GST_CAT_DEFAULT (rtpvorbispay_debug)
 
 /* references:
- * http://svn.xiph.org/trunk/vorbis/doc/draft-ietf-avt-rtp-vorbis-04.txt
+ * http://www.rfc-editor.org/rfc/rfc5215.txt
  */
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_vorbispay_details =
-GST_ELEMENT_DETAILS ("RTP packet depayloader",
+GST_ELEMENT_DETAILS ("RTP Vorbis depayloader",
     "Codec/Payloader/Network",
-    "Payload-encode Vorbis audio into RTP packets (draft-01 RFC XXXX)",
-    "Wim Taymans <wim@fluendo.com>");
+    "Payload-encode Vorbis audio into RTP packets (RFC 5215)",
+    "Wim Taymans <wimi.taymans@gmail.com>");
 
 static GstStaticPadTemplate gst_rtp_vorbis_pay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -55,11 +55,6 @@ GST_STATIC_PAD_TEMPLATE ("src",
          * "encoding-params = (string) <num channels>"
          * "configuration = (string) ANY"
          */
-        /* All optional parameters
-         *
-         * "delivery-method = (string) { inline, in_band, out_band/<specific_name> } "
-         * "configuration-uri ="
-         */
     )
     );
 
@@ -262,6 +257,7 @@ gst_rtp_vorbis_pay_finish_headers (GstBaseRTPPayload * basepayload)
   gchar *cstr, *configuration;
   guint8 *data, *config;
   guint32 ident;
+  gboolean res;
 
   GST_DEBUG_OBJECT (rtpvorbispay, "finish headers");
 
@@ -409,16 +405,13 @@ gst_rtp_vorbis_pay_finish_headers (GstBaseRTPPayload * basepayload)
   cstr = g_strdup_printf ("%d", rtpvorbispay->channels);
   gst_basertppayload_set_options (basepayload, "audio", TRUE, "VORBIS",
       rtpvorbispay->rate);
-  gst_basertppayload_set_outcaps (basepayload, "encoding-params", G_TYPE_STRING,
-      cstr, "configuration", G_TYPE_STRING, configuration,
-      "delivery-method", G_TYPE_STRING, "inline",
-      /* don't set the other defaults 
-       */
-      NULL);
+  res =
+      gst_basertppayload_set_outcaps (basepayload, "encoding-params",
+      G_TYPE_STRING, cstr, "configuration", G_TYPE_STRING, configuration, NULL);
   g_free (cstr);
   g_free (configuration);
 
-  return TRUE;
+  return res;
 
   /* ERRORS */
 no_headers:
