@@ -29,14 +29,9 @@
 #ifdef G_OS_WIN32
 /* ws2_32.dll has getaddrinfo and freeaddrinfo on Windows XP and later.
  * minwg32 headers check WINVER before allowing the use of these */
-#ifndef WINVER
 #define WINVER 0x0501
-#endif
 #include <winsock2.h>
 #include <ws2tcpip.h>
-#ifndef socklen_t
-#define socklen_t int
-#endif
 
 /* Needed for GstObject and GST_WARNING_OBJECT */
 #include <gst/gstobject.h>
@@ -44,10 +39,9 @@
 
 #else
 #include <sys/time.h>
-#include <sys/socket.h>
 #include <netinet/in.h>
-#include <net/if.h>
 #include <netdb.h>
+#include <sys/socket.h>
 #include <sys/wait.h>
 #include <arpa/inet.h>
 #include <unistd.h>
@@ -80,31 +74,15 @@ gboolean gst_udp_net_utils_win32_wsa_startup (GstObject * obj);
 
 #endif
 
-typedef struct {
-  gchar    *host;
-  gint      port;
-  gboolean  is_ipv6;
-} GstUDPUri;
+int gst_udp_get_sockaddr_length(struct sockaddr_storage *addr);
 
-int     gst_udp_get_sockaddr_length  (struct sockaddr_storage *addr);
+int gst_udp_get_addr      (const char *hostname, int port, struct sockaddr_storage *addr);
+int gst_udp_is_multicast  (struct sockaddr_storage *addr);
 
-int     gst_udp_get_addr             (const char *hostname, int port, struct sockaddr_storage *addr);
-int     gst_udp_is_multicast         (struct sockaddr_storage *addr);
+int gst_udp_set_loop_ttl  (int sockfd, gboolean loop, int ttl);
 
-int     gst_udp_set_loop             (int sockfd, guint16 ss_family, gboolean loop);
-int     gst_udp_set_ttl              (int sockfd, guint16 ss_family, int ttl, gboolean is_multicast);
-
-/* multicast groups */
-int     gst_udp_join_group           (int sockfd, struct sockaddr_storage *addr,
-                                      gchar *iface);
-int     gst_udp_leave_group          (int sockfd, struct sockaddr_storage *addr);
-
-/* uri handling */
-void    gst_udp_uri_init             (GstUDPUri *uri, const gchar *host, gint port);
-int     gst_udp_uri_update           (GstUDPUri *uri, const gchar *host, gint port);
-int     gst_udp_parse_uri            (const gchar *uristr, GstUDPUri *uri);
-gchar * gst_udp_uri_string           (GstUDPUri *uri);
-void    gst_udp_uri_free             (GstUDPUri *uri);
+int gst_udp_join_group    (int sockfd, struct sockaddr_storage *addr);
+int gst_udp_leave_group   (int sockfd, struct sockaddr_storage *addr);
 
 #endif /* __GST_UDP_NET_UTILS_H__*/
 
