@@ -22,72 +22,46 @@
  * SECTION:element-udpsrc
  * @see_also: udpsink, multifdsink
  *
- * <refsect2>
- * <para>
  * udpsrc is a network source that reads UDP packets from the network.
  * It can be combined with RTP depayloaders to implement RTP streaming.
- * </para>
- * <title>Examples</title>
- * <para>
- * Here is a simple pipeline to read from the default port and dump the udp packets.
- * <programlisting>
- * gst-launch -v udpsrc ! fakesink dump=1
- * </programlisting>
- * To actually generate udp packets on the default port one can use the
- * udpsink element. When running the following pipeline in another terminal, the
- * above mentioned pipeline should dump data packets to the console.
- * <programlisting>
- * gst-launch -v audiotestsrc ! udpsink
- * </programlisting>
- * </para>
- * <para>
+ *
  * The udpsrc element supports automatic port allocation by setting the
- * "port" property to 0. the following pipeline reads UDP from a free port.
- * <programlisting>
- * gst-launch -v udpsrc port=0 ! fakesink
- * </programlisting>
- * After setting the udpsrc to PAUSED, the allocated port can be obtained by
- * reading the port property.
- * </para>
- * <para>
- * udpsrc can read from multicast groups by setting the multicast_group property
- * to the IP address of the multicast group.
- * </para>
- * <para>
- * Alternatively one can provide a custom socket to udpsrc with the "sockfd" property,
- * udpsrc will then not allocate a socket itself but use the provided one.
- * </para>
- * <para>
- * The "caps" property is mainly used to give a type to the UDP packet so that they
- * can be autoplugged in GStreamer pipelines. This is very usefull for RTP 
- * implementations where the contents of the UDP packets is transfered out-of-bounds
- * using SDP or other means. 
- * </para>
- * <para>
- * The "buffer" property is used to change the default kernel buffer sizes used for
- * receiving packets. The buffer size may be increased for high-volume connections,
- * or may be decreased to limit the possible backlog of incoming data.
- * The system places an absolute limit on these values, on Linux, for example, the
- * default buffer size is typically 50K and can be increased to maximally 100K.
- * </para>
- * <para>
- * The "skip-first-bytes" property is used to strip off an arbitrary number of
- * bytes from the start of the raw udp packet and can be used to strip off
- * proprietary header, for example. 
- * </para>
- * <para>
- * The udpsrc is always a live source. It does however not provide a GstClock, this
- * is left for upstream elements such as an RTP session manager or demuxer (such
- * as an MPEG demuxer). As with all live sources, the captured buffers will have
- * their timestamp set to the current running time of the pipeline.
- * </para>
- * <para>
- * udpsrc implements a GstURIHandler interface that handles udp://host:port type
- * URIs.
- * </para>
- * <para>
- * If the <link linkend="GstUDPSrc--timeout">timeout property</link> is set to a
- * value bigger than 0, udpsrc will generate an element message named
+ * #GstUDPSrc:port property to 0. After setting the udpsrc to PAUSED, the
+ * allocated port can be obtained by reading the port property.
+ *
+ * udpsrc can read from multicast groups by setting the #GstUDPSrc:multicast-group
+ * property to the IP address of the multicast group.
+ *
+ * Alternatively one can provide a custom socket to udpsrc with the #GstUDPSrc:sockfd
+ * property, udpsrc will then not allocate a socket itself but use the provided
+ * one.
+ *
+ * The #GstUDPSrc:caps property is mainly used to give a type to the UDP packet
+ * so that they can be autoplugged in GStreamer pipelines. This is very usefull
+ * for RTP implementations where the contents of the UDP packets is transfered
+ * out-of-bounds using SDP or other means.
+ *
+ * The #GstUDPSrc:buffer property is used to change the default kernel buffer
+ * sizes used for receiving packets. The buffer size may be increased for
+ * high-volume connections, or may be decreased to limit the possible backlog of
+ * incoming data. The system places an absolute limit on these values, on Linux,
+ * for example, the default buffer size is typically 50K and can be increased to
+ * maximally 100K.
+ *
+ * The #GstUDPSrc:skip-first-bytes property is used to strip off an arbitrary
+ * number of bytes from the start of the raw udp packet and can be used to strip
+ * off proprietary header, for example. 
+ *
+ * The udpsrc is always a live source. It does however not provide a #GstClock,
+ * this is left for upstream elements such as an RTP session manager or demuxer
+ * (such as an MPEG demuxer). As with all live sources, the captured buffers
+ * will have their timestamp set to the current running time of the pipeline.
+ *
+ * udpsrc implements a #GstURIHandler interface that handles udp://host:port
+ * type URIs.
+ *
+ * If the #GstUDPSrc:timeout property is set to a value bigger than 0, udpsrc
+ * will generate an element message named
  * <classname>&quot;GstUDPSrcTimeout&quot;</classname>
  * if no data was recieved in the given timeout.
  * The message's structure contains one field:
@@ -105,15 +79,28 @@
  * </para>
  * <para>
  * A custom file descriptor can be configured with the 
- * <link linkend="GstUDPSrc--sockfd">sockfd property</link>. The socket will be
- * closed when setting the element to READY by default. This behaviour can be
- * overriden with the <link linkend="GstUDPSrc--closefd">closefd property</link>,
- * in which case the application is responsible for closing the file descriptor.
- * </para>
- * <para>
- * Last reviewed on 2007-09-20 (0.10.7)
- * </para>
+ * #GstUDPSrc:sockfd property. The socket will be closed when setting the
+ * element to READY by default. This behaviour can be
+ * overriden with the #GstUDPSrc:closefd property, in which case the application
+ * is responsible for closing the file descriptor.
+ *
+ * <refsect2>
+ * <title>Examples</title>
+ * |[
+ * gst-launch -v udpsrc ! fakesink dump=1
+ * ]| A pipeline to read from the default port and dump the udp packets.
+ * To actually generate udp packets on the default port one can use the
+ * udpsink element. When running the following pipeline in another terminal, the
+ * above mentioned pipeline should dump data packets to the console.
+ * |[
+ * gst-launch -v audiotestsrc ! udpsink
+ * ]|
+ * |[
+ * gst-launch -v udpsrc port=0 ! fakesink
+ * ]| read udp packets from a free port.
  * </refsect2>
+ *
+ * Last reviewed on 2007-09-20 (0.10.7)
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -141,13 +128,6 @@ typedef int socklen_t;
 GST_DEBUG_CATEGORY_STATIC (udpsrc_debug);
 #define GST_CAT_DEFAULT (udpsrc_debug)
 
-#define CLOSE_IF_REQUESTED(udpctx)                                        \
-G_STMT_START {                                                            \
-  if ((!udpctx->externalfd) || (udpctx->externalfd && udpctx->closefd))   \
-    CLOSE_SOCKET(udpctx->sock.fd);                                        \
-  udpctx->sock.fd = -1;                                                   \
-} G_STMT_END
-
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -191,6 +171,16 @@ enum
   PROP_LAST
 };
 
+#define CLOSE_IF_REQUESTED(udpctx)                                        \
+G_STMT_START {                                                            \
+  if ((!udpctx->externalfd) || (udpctx->externalfd && udpctx->closefd)) { \
+    CLOSE_SOCKET(udpctx->sock.fd);                                        \
+    if (udpctx->sock.fd == udpctx->sockfd)                                \
+      udpctx->sockfd = UDP_DEFAULT_SOCKFD;                                \
+  }                                                                       \
+  udpctx->sock.fd = UDP_DEFAULT_SOCK;                                     \
+} G_STMT_END
+
 static void gst_udpsrc_uri_handler_init (gpointer g_iface, gpointer iface_data);
 
 static GstCaps *gst_udpsrc_getcaps (GstBaseSrc * src);
@@ -244,9 +234,7 @@ static void
 gst_udpsrc_class_init (GstUDPSrcClass * klass)
 {
   GObjectClass *gobject_class;
-
   GstBaseSrcClass *gstbasesrc_class;
-
   GstPushSrcClass *gstpushsrc_class;
 
   gobject_class = (GObjectClass *) klass;
@@ -348,7 +336,10 @@ gst_udpsrc_finalize (GObject * object)
   g_free (udpsrc->multi_group);
   g_free (udpsrc->uri);
 
-  WSA_CLEANUP (src);
+  if (udpsrc->sockfd >= 0 && udpsrc->closefd)
+    CLOSE_SOCKET (udpsrc->sockfd);
+
+  WSA_CLEANUP (object);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -370,26 +361,18 @@ static GstFlowReturn
 gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
 {
   GstUDPSrc *udpsrc;
-
   GstNetBuffer *outbuf;
-
   struct sockaddr_storage tmpaddr;
-
   socklen_t len;
-
   guint8 *pktdata;
-
   gint pktsize;
-
 #ifdef G_OS_UNIX
   gint readsize;
 #elif defined G_OS_WIN32
   gulong readsize;
 #endif
   GstClockTime timeout;
-
   gint ret;
-
   gboolean try_again;
 
   udpsrc = GST_UDPSRC_CAST (psrc);
@@ -462,7 +445,11 @@ no_select:
 
   while (TRUE) {
     len = sizeof (struct sockaddr);
+#ifdef G_OS_WIN32
+    ret = recvfrom (udpsrc->sock.fd, (char *) pktdata, pktsize,
+#else
     ret = recvfrom (udpsrc->sock.fd, pktdata, pktsize,
+#endif
         0, (struct sockaddr *) &tmpaddr, &len);
     if (G_UNLIKELY (ret < 0)) {
 #ifdef G_OS_WIN32
@@ -584,9 +571,7 @@ static gboolean
 gst_udpsrc_set_uri (GstUDPSrc * src, const gchar * uri)
 {
   gchar *protocol;
-
   gchar *location;
-
   gchar *colptr;
 
   protocol = gst_uri_get_protocol (uri);
@@ -671,6 +656,9 @@ gst_udpsrc_set_property (GObject * object, guint prop_id, const GValue * value,
       break;
     }
     case PROP_SOCKFD:
+      if (udpsrc->sockfd >= 0 && udpsrc->sockfd != udpsrc->sock.fd &&
+          udpsrc->closefd)
+        CLOSE_SOCKET (udpsrc->sockfd);
       udpsrc->sockfd = g_value_get_int (value);
       GST_DEBUG ("setting SOCKFD to %d", udpsrc->sockfd);
       break;
@@ -742,19 +730,16 @@ static gboolean
 gst_udpsrc_start (GstBaseSrc * bsrc)
 {
   guint bc_val;
-
   gint reuse;
-
   int port;
-
   GstUDPSrc *src;
-
   gint ret;
-
   int rcvsize;
-
+#ifdef G_OS_WIN32
+  gint len;
+#else
   guint len;
-
+#endif
   src = GST_UDPSRC (bsrc);
 
   if (src->sockfd == -1) {
@@ -778,8 +763,9 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
       goto setsockopt_error;
 
     GST_DEBUG_OBJECT (src, "binding on port %d", src->port);
-    if ((ret = bind (src->sock.fd, (struct sockaddr *) &src->myaddr,
-                sizeof (src->myaddr))) < 0)
+
+    len = gst_udp_get_sockaddr_length (&src->myaddr);
+    if ((ret = bind (src->sock.fd, (struct sockaddr *) &src->myaddr, len)) < 0)
       goto bind_error;
 
     len = sizeof (src->myaddr);
