@@ -50,6 +50,12 @@
 GST_DEBUG_CATEGORY_EXTERN (sunaudio_debug);
 #define GST_CAT_DEFAULT sunaudio_debug
 
+static GstElementDetails plugin_details =
+GST_ELEMENT_DETAILS ("Sun Audio Source",
+    "Source/Audio",
+    "Audio source for Sun Audio devices",
+    "Brian Cameron <brian.cameron@sun.com>");
+
 static void gst_sunaudiosrc_base_init (gpointer g_class);
 static void gst_sunaudiosrc_class_init (GstSunAudioSrcClass * klass);
 static void gst_sunaudiosrc_init (GstSunAudioSrc * sunaudiosrc,
@@ -108,12 +114,9 @@ gst_sunaudiosrc_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_sunaudiosrc_factory);
-  gst_element_class_set_details_simple (element_class, "Sun Audio Source",
-      "Source/Audio",
-      "Audio source for Sun Audio devices",
-      "Brian Cameron <brian.cameron@sun.com>");
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_sunaudiosrc_factory));
+  gst_element_class_set_details (element_class, &plugin_details);
 }
 
 static void
@@ -131,9 +134,11 @@ gst_sunaudiosrc_class_init (GstSunAudioSrcClass * klass)
   gstbaseaudiosrc_class = (GstBaseAudioSrcClass *) klass;
   gstaudiosrc_class = (GstAudioSrcClass *) klass;
 
-  gobject_class->dispose = gst_sunaudiosrc_dispose;
-  gobject_class->get_property = gst_sunaudiosrc_get_property;
-  gobject_class->set_property = gst_sunaudiosrc_set_property;
+  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_sunaudiosrc_dispose);
+  gobject_class->get_property =
+      GST_DEBUG_FUNCPTR (gst_sunaudiosrc_get_property);
+  gobject_class->set_property =
+      GST_DEBUG_FUNCPTR (gst_sunaudiosrc_set_property);
 
   gstbasesrc_class->get_caps = GST_DEBUG_FUNCPTR (gst_sunaudiosrc_getcaps);
 
@@ -148,7 +153,7 @@ gst_sunaudiosrc_class_init (GstSunAudioSrcClass * klass)
   g_object_class_install_property (gobject_class, PROP_DEVICE,
       g_param_spec_string ("device", "Device",
           "SunAudio device (usually /dev/audio)", DEFAULT_DEVICE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
 }
 
 static void
@@ -307,8 +312,8 @@ gst_sunaudiosrc_prepare (GstAudioSrc * asrc, GstRingBufferSpec * spec)
   GstSunAudioSrc *sunaudiosrc = GST_SUNAUDIO_SRC (asrc);
   audio_info_t ainfo;
   int ret;
-  GstSunAudioMixerCtrl *mixer;
-  struct audio_info audioinfo;
+  int ctrl_fd = -1;
+  int ports;
 
   ret = ioctl (sunaudiosrc->fd, AUDIO_GETINFO, &ainfo);
   if (ret == -1) {
@@ -328,7 +333,8 @@ gst_sunaudiosrc_prepare (GstAudioSrc * asrc, GstRingBufferSpec * spec)
   ainfo.record.encoding = AUDIO_ENCODING_LINEAR;
   ainfo.record.buffer_size = spec->buffer_time;
 
-  mixer = sunaudiosrc->mixer;
+  GstSunAudioMixerCtrl *mixer = sunaudiosrc->mixer;
+  struct audio_info audioinfo;
 
   if (ioctl (mixer->mixer_fd, AUDIO_GETINFO, &audioinfo) < 0) {
     g_warning ("Error getting audio device volume");
